# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-09-18 13:09
Short Description:

Change History:

'''


def remove_key_if_exists(info_dict: dict, rm_list: list):
    for rm_name in rm_list:
        if rm_name in info_dict:
            del info_dict[rm_name]
    return info_dict


def get_name_without_suffix_from_path(path_like, ignore=False):
    # print(path_like)
    name = path_like.split("/")[-1]
    if ignore == False:
        assert name.count(".") == 1, "文件名存在多个.后缀或者没有.，请确认处理方式"
        name_without_suffix = name.split(".")[0]
    else:
        name_without_suffix = '.'.join(name.split(".")[:-1])
    return name_without_suffix



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-01 15:33
Short Description:

Change History:


主要用于预处理的数据转成平台的格式,平台最少的数据格式
注意单帧合并成连续帧的时候，id不同，number也不能一样， id和number必须一一对应的。(特别是全局编号的模式下，id和number是一一对应的)

{

	"frames": [{
		"frameId": 0,
		"items": [{
			"id": "669b1f5a-8eeb-40fe-8945-2dd8cec688a1",
			"number": 1,
			"position": {
				"x": -2.9164659842583536,
				"y": -51.38931599127634,
				"z": -1.8388888888888877
			},
			"rotation": {
				"x": 0,
				"y": 0,
				"z": -1.6618072479377624
			},
			"dimension": {
				"x": 4.2,
				"y": 1.8,
				"z": 1.6
			},
			"category": "轿车",
			"labels": "...json字符串..."
		}],
		"images": [
			[{
				"items": [{
					"type": "RECT",
					"id": "669b1f5a-8eeb-40fe-8945-2dd8cec688a1",
					"number": 1,
					"category": "轿车",
					"position": {
						"x": 944.9714187283353,
						"y": 712.9711061567027
					},
					"dimension": {
						"x": 71.2262093020197,
						"y": 58.792783067844766
					},
					"labels": "...json字符串..."

				}]
			}]
		]

	}]

}

'''
from peutils.transform.v1.base import *
import json




class LidarBoxPre():
    ### 输入连续帧数量
    ### 镜头顺序，可根据新数据的manifest读取

    def __init__(self,frame_length,camera_list):
        self.frame_length = frame_length
        self.camera_list = camera_list  # 镜头顺序，可以从新的数据中取到。如果变换了之后可能会影响到数据的
        self.frame_list = [{
                                "frameId":i,
                                "items":[],
                                "images":[
                                    {
                                        "items":[]
                                    }
                                          for j in range(len(self.camera_list))]
                            } for i in range(self.frame_length)]  # 写完后的完整数据

    def add_lidar_obj(self,frameNum,lidar_obj):
        self.frame_list[frameNum]["items"].append(
            lidar_obj.to_dict()
        )

    def add_img_obj(self,frameNum,camera,img_obj):
        cam_idx = self.camera_list.index(camera)
        self.frame_list[frameNum]["images"][cam_idx]["items"].append(
            img_obj.to_dict()
        )

    def dumps_data(self):
        out_dict = {
            "frames":self.frame_list
        }
        return json.dumps(out_dict,ensure_ascii=False)



if __name__ =="__main__":
    pass

    # from peutils.transform.v1.lidar_manifest.parser import LidarManifestParse,LidarManifestConfig
    # msft = LidarManifestParse(base_url="https://projecteng.oss-cn-shanghai.aliyuncs.com/3d_json/test_shuima/2022-05-20-09-38-20-04/json/2022-05-20-09-38-20-04_1_50",
    #                           config=LidarManifestConfig())
    # ldboxpre2 = LidarBoxPre(frame_length=msft.frame_length,camera_list=msft.camera_list)
    #
    # from peutils.transform.v1.lidar_box.parser import LidarBoxParse,LidarBoxDataConfig
    # parse_obj = LidarBoxParse(url="https://projecteng.oss-cn-shanghai.aliyuncs.com/3d_json/test_shuima/2022-05-20-09-38-20-04/R.1655858875766.202.19671_2022-06-21T202615Z.599.QA_RW.27895.0f0425a96442183be87772da04b197a0.review.json",
    #                               config=LidarBoxDataConfig(
    #                                   yaw_only=False,
    #                                   has_pointCount=False,
    #                                   number_adpter_func=None,  # lambda i: round(i,3), # 默认None
    #                                   parse_id_col="id",
    #                                   seq_start=0,  # 如果是 gid,fid,或者frame_id 需要提供seq_start， 如果是0就是从1开始编号，如果是-1就是从0开始编号
    #                                   overflow=True
    #                               )
    # )
    # for _,lidar in parse_obj.frames_lst[0].lidar_dict.items():
    #     # lidar.p
    #     ldboxpre2.add_lidar_obj(frameNum=0,lidar_obj=lidar)
    # for camera, img_dict in parse_obj.frames_lst[0].images_dict.items():  ##注意 这个cemra不一定准确
    #     for k,v in img_dict.items():
    #     # print(camera,img)
    #         ldboxpre2.add_img_obj(frameNum=0, camera=camera, img_obj=v)
    # # print(ldboxpre2)
    # print(ldboxpre2.dumps_data())


    # ldboxpre = LidarBoxPre(frame_length = 50,camera_list=['front_middle_camera', 'lf_wide_camera', 'lr_wide_camera', 'rear_middle_camera', 'rf_wide_camera', 'rr_wide_camera'])
    # print(ldboxpre.dumps_data())

    # self.frames = {}





# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-03 06:38
Short Description:

Change History:

'''
from peutils.transform.v1.lidar_box.parser import LidarBoxParse,LidarBoxDataConfig
from peutils.transform.v1.task.executor import AbstractTask,CommonTaskV1

import logging
logging.basicConfig(level=logging.INFO)
import time
import random



class AfterParse(AbstractTask):

    def __init__(self,row,users_param=None):
        super().__init__(row,users_param)
        self.parse_obj = LidarBoxParse(row["annotation"], config=LidarBoxDataConfig(
                             yaw_only = True,
                             has_pointCount= True,
                             number_adpter_func=None, #lambda i: round(i,3), # 默认None
                             parse_id_col = "fid",
                             overflow= False
                         ))
        self.log.error_list.extend(self.parse_obj.check_frames_error())


    def check_row_on_a9(self) ->None:
        pass


        # self.log.create_error("发现错误")

    def check_row_addtional(self) ->None:
        pass




    def parse_row_func(self) ->None:
        logging.info("s")
        # print(self.log.error_list)
        # if len(self.log.error_list) >0:
        #     logging.error(self.log.error_list)
        # logging.error(self.log.error_list)

        # logging.debug("haha")
        # time.sleep(random.randint(1,10))
        # logging.info("_______ok_________ ")
        # from pprint import pprint
        # pprint(self.parse_obj.frames_lst[0].lidar_dict)

        # upload_group = self.row["upload_group"]
        # for frame in self.parse_obj.frames_lst:
        #     lidar_template = {
        #         "objects": []
        #     }
        #     for _,v in frame.lidar_dict.items():
        #         obj = {
        #             "id":""
        #         }





        # print(self.parse_obj.frames_lst)


        # print("_______ok_________")

#
# af = AfterParse(
#                 row={
#     "annotation":"http://oss.prd.appen.com.cn:9000/appen-lidar-prod/a019f71166e17533eb96741ebd0f0d0a/R.1653455556980.9f873b2b-71a8-4e85-9fef-9ec33e87d806.CMyfqgIQDw3d3d_2022-05-25T050329Z.49643.QA_RW.8b67819f-e6ad-4a1d-a087-1f35646a05fc.67f6a851b7317a4a1840f0b77f66703c.review.json"
# })
# af.parse_row_func()

### 并发执行
ct = CommonTaskV1(AfterParse)
ct.process_func("/Users/hwang2/Desktop/Result_2022-06-07T06_15_01.489694Z.csv",k1="v1",k2="v2")






# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-04-21 16:11
Short Description:

Change History:

'''



def rectToBbox(shape:dict,adapter=None):
    '''
    平台的rectangle 类型转成 [x_min, y_min, w, h]
    如果是先取证，再转的话是 lambda i: int(round(i,0))
    :param shape:
    :param adapter:
    :return:
    '''
    points = shape['points']
    width = shape['width']
    height = shape["height"]
    assert width is not None and height is not None and points is not None,"rectangle长宽高和点不能为None"

    left_top = min(points,key=lambda i: (i["x"],i["y"]))
    x = left_top["x"]
    y = left_top["y"]

    if adapter is None:
        return x,y ,width,height
    else:
        return adapter(x),adapter(y),adapter(width),adapter(height)


def pointsToList(points:list,adpter=None):
    assert points is not None,"points不能为None"

    if adpter is None:
        return [[ p["x"], p["y"]] for p in points]
    else:
        return  [[ adpter(p["x"]), adpter(p["y"])] for p in points]





# -*- coding: utf-8 -*-
# @Author : Thomas
# @File : pcd_py3.py
# @CreateTime : 2021/6/4 11:51 上午
# @Project: 
# @Description :

"""
Read and write PCL .pcd files in python.
dimatura@cmu.edu, 2013-2018
- TODO better API for wacky operations.
- TODO add a cli for common operations.
- TODO deal properly with padding
- TODO deal properly with multicount fields
- TODO better support for rgb nonsense
"""

import re
import struct
import copy
# import cStringIO as sio
# from io import StringIO as sio
from io import BytesIO
import numpy as np
import warnings
import lzf
import uuid



HAS_SENSOR_MSGS = True
try:
    from sensor_msgs.msg import PointField
    import numpy_pc2  # needs sensor_msgs
except ImportError:
    HAS_SENSOR_MSGS = False

__all__ = ['PointCloud',
           # 'point_cloud_to_path',
           # 'point_cloud_to_buffer',
           # 'point_cloud_to_fileobj',
           'point_cloud_from_path',
           'point_cloud_from_buffer',
           'point_cloud_from_fileobj',
           # 'make_xyz_point_cloud',
           # 'make_xyz_rgb_point_cloud',
           # 'make_xyz_label_point_cloud',
           # 'save_txt',
           # 'cat_point_clouds',
           # 'add_fields',
           # 'update_field',
           # 'build_ascii_fmtstr',
           # 'encode_rgb_for_pcl',
           # 'decode_rgb_from_pcl',
           # 'save_point_cloud',
           # 'save_point_cloud_bin',
           # 'save_point_cloud_bin_compressed',
           # 'pcd_type_to_numpy_type',
           # 'numpy_type_to_pcd_type',
           ]

if HAS_SENSOR_MSGS:
    pc2_pcd_type_mappings = [(PointField.INT8, ('I', 1)),
                             (PointField.UINT8, ('U', 1)),
                             (PointField.INT16, ('I', 2)),
                             (PointField.UINT16, ('U', 2)),
                             (PointField.INT32, ('I', 4)),
                             (PointField.UINT32, ('U', 4)),
                             (PointField.FLOAT32, ('F', 4)),
                             (PointField.FLOAT64, ('F', 8))]
    pc2_type_to_pcd_type = dict(pc2_pcd_type_mappings)
    pcd_type_to_pc2_type = dict((q, p) for (p, q) in pc2_pcd_type_mappings)
    __all__.extend(['pcd_type_to_pc2_type', 'pc2_type_to_pcd_type'])

numpy_pcd_type_mappings = [(np.dtype('float32'), ('F', 4)),
                           (np.dtype('float64'), ('F', 8)),
                           (np.dtype('uint8'), ('U', 1)),
                           (np.dtype('uint16'), ('U', 2)),
                           (np.dtype('uint32'), ('U', 4)),
                           (np.dtype('uint64'), ('U', 8)),
                           (np.dtype('int16'), ('I', 2)),
                           (np.dtype('int32'), ('I', 4)),
                           (np.dtype('int64'), ('I', 8))]
numpy_type_to_pcd_type = dict(numpy_pcd_type_mappings)
pcd_type_to_numpy_type = dict((q, p) for (p, q) in numpy_pcd_type_mappings)


def parse_header(lines):
    """ Parse header of PCD files.
    """
    metadata = {}
    for ln in lines:
        if ln.startswith('#') or len(ln) < 2:
            continue
        match = re.match('(\w+)\s+([\w\s\.]+)', ln)
        if not match:
            warnings.warn("warning: can't understand line: %s" % ln)
            continue
        key, value = match.group(1).lower(), match.group(2)
        if key == 'version':
            metadata[key] = value
        # modify by thomas for duplicated fields start
        elif key == 'type':
            metadata[key] = value.split()
        elif key == 'fields':
            # metadata[key] = value.split()
            origin_fields = value.split()
            new_fields = []
            for field in origin_fields:
                if field not in new_fields:
                    new_fields.append(field)
                else:
                    new_fields.append("".join([field, "-", str(uuid.uuid4())]))
            metadata[key] = new_fields
        # modify by thomas for duplicated fields end
        elif key in ('size', 'count'):
            # metadata[key] = map(int, value.split())  modify by thomas for python3 compatible
            metadata[key] = list(map(int, value.split()))
        elif key in ('width', 'height', 'points'):
            metadata[key] = int(value)
        elif key == 'viewpoint':
            # metadata[key] = map(float, value.split())  modify by thomas
            metadata[key] = list(map(float, value.split()))
        elif key == 'data':
            metadata[key] = value.strip().lower()
        # TODO apparently count is not required?
    # add some reasonable defaults
    if 'count' not in metadata:
        metadata['count'] = [1]*len(metadata['fields'])
    if 'viewpoint' not in metadata:
        metadata['viewpoint'] = [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0]
    if 'version' not in metadata:
        metadata['version'] = '.7'
    return metadata


def write_header(metadata, rename_padding=False):
    """ Given metadata as dictionary, return a string header.
    """
    template = """\
VERSION {version}
FIELDS {fields}
SIZE {size}
TYPE {type}
COUNT {count}
WIDTH {width}
HEIGHT {height}
VIEWPOINT {viewpoint}
POINTS {points}
DATA {data}
"""
    str_metadata = metadata.copy()

    if not rename_padding:
        str_metadata['fields'] = ' '.join(metadata['fields'])
    else:
        new_fields = []
        for f in metadata['fields']:
            if f == '_':
                new_fields.append('padding')
            else:
                new_fields.append(f)
        str_metadata['fields'] = ' '.join(new_fields)
    str_metadata['size'] = ' '.join(map(str, metadata['size']))
    str_metadata['type'] = ' '.join(metadata['type'])
    str_metadata['count'] = ' '.join(map(str, metadata['count']))
    str_metadata['width'] = str(metadata['width'])
    str_metadata['height'] = str(metadata['height'])
    str_metadata['viewpoint'] = ' '.join(map(str, metadata['viewpoint']))
    str_metadata['points'] = str(metadata['points'])
    tmpl = template.format(**str_metadata)
    return tmpl


def _metadata_is_consistent(metadata):
    """ Sanity check for metadata. Just some basic checks.
    """
    checks = []
    required = ('version', 'fields', 'size', 'width', 'height', 'points',
                'viewpoint', 'data')
    for f in required:
        if f not in metadata:
            print('%s required' % f)
    checks.append((lambda m: all([k in m for k in required]),
                   'missing field'))
    checks.append((lambda m: len(m['type']) == len(list(m['count'])) ==
                   len(m['fields']),
                   'length of type, count and fields must be equal'))
    checks.append((lambda m: m['height'] > 0,
                   'height must be greater than 0'))
    checks.append((lambda m: m['width'] > 0,
                   'width must be greater than 0'))
    checks.append((lambda m: m['points'] > 0,
                   'points must be greater than 0'))
    checks.append((lambda m: m['data'].lower() in ('ascii', 'binary',
                   'binary_compressed'),
                   'unknown data type:'
                   'should be ascii/binary/binary_compressed'))
    ok = True
    for check, msg in checks:
        if not check(metadata):
            print('error:', msg)
            ok = False
    return ok

# def pcd_type_to_numpy(pcd_type, pcd_sz):
#     """ convert from a pcd type string and size to numpy dtype."""
#     typedict = {'F' : { 4:np.float32, 8:np.float64 },
#                 'I' : { 1:np.int8, 2:np.int16, 4:np.int32, 8:np.int64 },
#                 'U' : { 1:np.uint8, 2:np.uint16, 4:np.uint32 , 8:np.uint64 }}
#     return typedict[pcd_type][pcd_sz]


def _build_dtype(metadata):
    """ Build numpy structured array dtype from pcl metadata.
    Note that fields with count > 1 are 'flattened' by creating multiple
    single-count fields.
    *TODO* allow 'proper' multi-count fields.
    """
    fieldnames = []
    typenames = []
    for f, c, t, s in zip(metadata['fields'],
                          metadata['count'],
                          metadata['type'],
                          metadata['size']):
        np_type = pcd_type_to_numpy_type[(t, s)]
        if c == 1:
            fieldnames.append(f)
            typenames.append(np_type)
        else:
            # fieldnames.extend(['%s_%04d' % (f, i) for i in xrange(c)])  modify by thomas
            fieldnames.extend(['%s_%04d' % (f, i) for i in range(c)])
            typenames.extend([np_type]*c)
    # dtype = np.dtype(zip(fieldnames, typenames))  modify by thomas
    dtype = np.dtype([(f, t) for f, t in zip(fieldnames, typenames)])
    return dtype


def build_ascii_fmtstr(pc):
    """ Make a format string for printing to ascii.
    Note %.8f is minimum for rgb.
    """
    fmtstr = []
    for t, cnt in zip(pc.type, pc.count):
        if t == 'F':
            fmtstr.extend(['%.10f']*cnt)
        elif t == 'I':
            fmtstr.extend(['%d']*cnt)
        elif t == 'U':
            fmtstr.extend(['%u']*cnt)
        else:
            raise ValueError("don't know about type %s" % t)
    return fmtstr


def parse_ascii_pc_data(f, dtype, metadata):
    """ Use numpy to parse ascii pointcloud data.
    """
    return np.loadtxt(f, dtype=dtype, delimiter=' ')


def parse_binary_pc_data(f, dtype, metadata):
    rowstep = metadata['points']*dtype.itemsize
    # for some reason pcl adds empty space at the end of files
    buf = f.read(rowstep)
    return np.fromstring(buf, dtype=dtype)


def parse_binary_compressed_pc_data(f, dtype, metadata):
    """ Parse lzf-compressed data.
    Format is undocumented but seems to be:
    - compressed size of data (uint32)
    - uncompressed size of data (uint32)
    - compressed data
    - junk
    """
    fmt = 'II'
    compressed_size, uncompressed_size =\
        struct.unpack(fmt, f.read(struct.calcsize(fmt)))
    compressed_data = f.read(compressed_size)
    # TODO what to use as second argument? if buf is None
    # (compressed > uncompressed)
    # should we read buf as raw binary?
    buf = lzf.decompress(compressed_data, uncompressed_size)
    if len(buf) != uncompressed_size:
        raise IOError('Error decompressing data')

    # modify by thomas for mulit height start
    # the data is stored field-by-field
    # pc_data = np.zeros(metadata['width'], dtype=dtype)
    # ix = 0
    # for dti in range(len(dtype)):
    #     dt = dtype[dti]
    #     bytes = dt.itemsize * metadata['width']
    #     column = np.fromstring(buf[ix:(ix+bytes)], dt)
    #     pc_data[dtype.names[dti]] = column
    #     ix += bytes
    # return pc_data
    pc_data = np.zeros(metadata['width']*metadata['height'], dtype=dtype)
    ix = 0
    for dti in range(len(dtype)):
        dt = dtype[dti]
        bytes = dt.itemsize * metadata['width'] * metadata['height']
        column = np.fromstring(buf[ix:(ix+bytes)], dt)
        pc_data[dtype.names[dti]] = column
        ix += bytes
    return pc_data
    # modify by thomas for mulit height end


def point_cloud_from_fileobj(f):
    """ Parse pointcloud coming from file object f
    """
    header = []
    while True:
        # ln = f.readline().strip()  modify by thomas
        ln = f.readline().strip().decode()
        header.append(ln)
        if ln.startswith('DATA'):
            metadata = parse_header(header)
            dtype = _build_dtype(metadata)
            break
    if metadata['data'] == 'ascii':
        pc_data = parse_ascii_pc_data(f, dtype, metadata)
    elif metadata['data'] == 'binary':
        pc_data = parse_binary_pc_data(f, dtype, metadata)
    elif metadata['data'] == 'binary_compressed':
        pc_data = parse_binary_compressed_pc_data(f, dtype, metadata)
    else:
        print('DATA field is neither "ascii" or "binary" or\
                "binary_compressed"')
    return PointCloud(metadata, pc_data)


def point_cloud_from_path(fname):
    """ load point cloud in binary format
    """
    with open(fname, 'rb') as f:
        pc = point_cloud_from_fileobj(f)
    return pc


def point_cloud_from_buffer(buf):
    fileobj = BytesIO(buf)
    pc = point_cloud_from_fileobj(fileobj)
    fileobj.close()  # necessary?
    return pc


def point_cloud_to_fileobj(pc, fileobj, data_compression=None):
    """ Write pointcloud as .pcd to fileobj.
    If data_compression is not None it overrides pc.data.
    """
    metadata = pc.get_metadata()
    if data_compression is not None:
        data_compression = data_compression.lower()
        assert(data_compression in ('ascii', 'binary', 'binary_compressed'))
        metadata['data'] = data_compression

    header = write_header(metadata)
    fileobj.write(header)
    if metadata['data'].lower() == 'ascii':
        fmtstr = build_ascii_fmtstr(pc)
        np.savetxt(fileobj, pc.pc_data, fmt=fmtstr)
    elif metadata['data'].lower() == 'binary':
        fileobj.write(pc.pc_data.tostring('C'))
    elif metadata['data'].lower() == 'binary_compressed':
        # TODO
        # a '_' field is ignored by pcl and breakes compressed point clouds.
        # changing '_' to '_padding' or other name fixes this.
        # admittedly padding shouldn't be compressed in the first place.
        # reorder to column-by-column
        uncompressed_lst = []
        for fieldname in pc.pc_data.dtype.names:
            column = np.ascontiguousarray(pc.pc_data[fieldname]).tostring('C')
            uncompressed_lst.append(column)
        uncompressed = ''.join(uncompressed_lst)
        uncompressed_size = len(uncompressed)
        # print("uncompressed_size = %r"%(uncompressed_size))
        buf = lzf.compress(uncompressed)
        if buf is None:
            # compression didn't shrink the file
            # TODO what do to do in this case when reading?
            buf = uncompressed
            compressed_size = uncompressed_size
        else:
            compressed_size = len(buf)
        fmt = 'II'
        fileobj.write(struct.pack(fmt, compressed_size, uncompressed_size))
        fileobj.write(buf)
    else:
        raise ValueError('unknown DATA type')
    # we can't close because if it's stringio buf then we can't get value after


# def point_cloud_to_path(pc, fname):
#     with open(fname, 'w') as f:
#         point_cloud_to_fileobj(pc, f)
#
#
# def point_cloud_to_buffer(pc, data_compression=None):
#     fileobj = sio.StringIO()
#     point_cloud_to_fileobj(pc, fileobj, data_compression)
#     return fileobj.getvalue()
#
#
# def save_point_cloud(pc, fname):
#     """ Save pointcloud to fname in ascii format.
#     """
#     with open(fname, 'w') as f:
#         point_cloud_to_fileobj(pc, f, 'ascii')
#
#
# def save_point_cloud_bin(pc, fname):
#     """ Save pointcloud to fname in binary format.
#     """
#     with open(fname, 'w') as f:
#         point_cloud_to_fileobj(pc, f, 'binary')
#
#
# def save_point_cloud_bin_compressed(pc, fname):
#     """ Save pointcloud to fname in binary compressed format.
#     """
#     with open(fname, 'w') as f:
#         point_cloud_to_fileobj(pc, f, 'binary_compressed')
#
#
# def save_xyz_label(pc, fname, use_default_lbl=False):
#     """ Save a simple (x y z label) pointcloud, ignoring all other features.
#     Label is initialized to 1000, for an obscure program I use.
#     """
#     md = pc.get_metadata()
#     if not use_default_lbl and ('label' not in md['fields']):
#         raise Exception('label is not a field in this point cloud')
#     with open(fname, 'w') as f:
#         # for i in xrange(pc.points): modify by thomas
#         for i in range(pc.points):
#             x, y, z = ['%.4f' % d for d in (
#                 pc.pc_data['x'][i], pc.pc_data['y'][i], pc.pc_data['z'][i]
#                 )]
#             lbl = '1000' if use_default_lbl else pc.pc_data['label'][i]
#             f.write(' '.join((x, y, z, lbl))+'\n')
#
#
# def save_xyz_intensity_label(pc, fname, use_default_lbl=False):
#     """ Save XYZI point cloud.
#     """
#     md = pc.get_metadata()
#     if not use_default_lbl and ('label' not in md['fields']):
#         raise Exception('label is not a field in this point cloud')
#     if 'intensity' not in md['fields']:
#         raise Exception('intensity is not a field in this point cloud')
#     with open(fname, 'w') as f:
#         # for i in xrange(pc.points): modify by thomas
#         for i in range(pc.points):
#             x, y, z = ['%.4f' % d for d in (
#                 pc.pc_data['x'][i], pc.pc_data['y'][i], pc.pc_data['z'][i]
#                 )]
#             intensity = '%.4f' % pc.pc_data['intensity'][i]
#             lbl = '1000' if use_default_lbl else pc.pc_data['label'][i]
#             f.write(' '.join((x, y, z, intensity, lbl))+'\n')
#
#
# def save_txt(pc, fname, header=True):
#     """ Save to csv-style text file, separated by spaces.
#     TODO:
#     - support multi-count fields.
#     - other delimiters.
#     """
#     with open(fname, 'w') as f:
#         if header:
#             header_lst = []
#             for field_name, cnt in zip(pc.fields, pc.count):
#                 if cnt == 1:
#                     header_lst.append(field_name)
#                 else:
#                     # for c in xrange(cnt): modify by thomas
#                     for c in range(cnt):
#                         header_lst.append('%s_%04d' % (field_name, c))
#             f.write(' '.join(header_lst)+'\n')
#         fmtstr = build_ascii_fmtstr(pc)
#         np.savetxt(f, pc.pc_data, fmt=fmtstr)
#
#
# def update_field(pc, field, pc_data):
#     """ Updates field in-place.
#     """
#     pc.pc_data[field] = pc_data
#     return pc
#
#
# def add_fields(pc, metadata, pc_data):
#     """ Builds copy of pointcloud with extra fields.
#     Multi-count fields are sketchy, yet again.
#     """
#     if len(set(metadata['fields']).intersection(set(pc.fields))) > 0:
#         raise Exception("Fields with that name exist.")
#
#     if pc.points != len(pc_data):
#         raise Exception("Mismatch in number of points.")
#
#     new_metadata = pc.get_metadata()
#     new_metadata['fields'].extend(metadata['fields'])
#     new_metadata['count'].extend(metadata['count'])
#     new_metadata['size'].extend(metadata['size'])
#     new_metadata['type'].extend(metadata['type'])
#
#     # parse metadata to add
#     # TODO factor this
#     fieldnames, typenames = [], []
#     for f, c, t, s in zip(metadata['fields'],
#                           metadata['count'],
#                           metadata['type'],
#                           metadata['size']):
#         np_type = pcd_type_to_numpy_type[(t, s)]
#         if c == 1:
#             fieldnames.append(f)
#             typenames.append(np_type)
#         else:
#             # fieldnames.extend(['%s_%04d' % (f, i) for i in xrange(c)]) modify by thomas
#             fieldnames.extend(['%s_%04d' % (f, i) for i in range(c)])
#             typenames.extend([np_type]*c)
#     dtype = zip(fieldnames, typenames)
#     # new dtype. could be inferred?
#     new_dtype = [(f, pc.pc_data.dtype[f])
#                  for f in pc.pc_data.dtype.names] + dtype
#
#     new_data = np.empty(len(pc.pc_data), new_dtype)
#     for n in pc.pc_data.dtype.names:
#         new_data[n] = pc.pc_data[n]
#     for n, n_tmp in zip(fieldnames, pc_data.dtype.names):
#         new_data[n] = pc_data[n_tmp]
#
#     # TODO maybe just all the metadata in the dtype.
#     # TODO maybe use composite structured arrays for fields with count > 1
#     newpc = PointCloud(new_metadata, new_data)
#     return newpc
#
#
# def cat_point_clouds(pc1, pc2):
#     """ Concatenate two point clouds into bigger point cloud.
#     Point clouds must have same metadata.
#     """
#     if len(pc1.fields) != len(pc2.fields):
#         raise ValueError("Pointclouds must have same fields")
#     new_metadata = pc1.get_metadata()
#     new_data = np.concatenate((pc1.pc_data, pc2.pc_data))
#     # TODO this only makes sense for unstructured pc?
#     new_metadata['width'] = pc1.width+pc2.width
#     new_metadata['points'] = pc1.points+pc2.points
#     pc3 = PointCloud(new_metadata, new_data)
#     return pc3
#
#
# def make_xyz_point_cloud(xyz, metadata=None):
#     """ Make a pointcloud object from xyz array.
#     xyz array is cast to float32.
#     """
#     md = {'version': .7,
#           'fields': ['x', 'y', 'z'],
#           'size': [4, 4, 4],
#           'type': ['F', 'F', 'F'],
#           'count': [1, 1, 1],
#           'width': len(xyz),
#           'height': 1,
#           'viewpoint': [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
#           'points': len(xyz),
#           'data': 'binary'}
#     if metadata is not None:
#         md.update(metadata)
#     xyz = xyz.astype(np.float32)
#     pc_data = xyz.view(np.dtype([('x', np.float32),
#                                  ('y', np.float32),
#                                  ('z', np.float32)]))
#     # pc_data = np.rec.fromarrays([xyz[:,0], xyz[:,1], xyz[:,2]], dtype=dt)
#     # data = np.rec.fromarrays([xyz.T], dtype=dt)
#     pc = PointCloud(md, pc_data)
#     return pc
#
#
# def make_xyz_rgb_point_cloud(xyz_rgb, metadata=None):
#     """ Make a pointcloud object from xyz array.
#     xyz array is assumed to be float32.
#     rgb is assumed to be encoded as float32 according to pcl conventions.
#     """
#     md = {'version': .7,
#           'fields': ['x', 'y', 'z', 'rgb'],
#           'count': [1, 1, 1, 1],
#           'width': len(xyz_rgb),
#           'height': 1,
#           'viewpoint': [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
#           'points': len(xyz_rgb),
#           'type': ['F', 'F', 'F', 'F'],
#           'size': [4, 4, 4, 4],
#           'data': 'binary'}
#     if xyz_rgb.dtype != np.float32:
#         raise ValueError('array must be float32')
#     if metadata is not None:
#         md.update(metadata)
#     pc_data = xyz_rgb.view(np.dtype([('x', np.float32),
#                                      ('y', np.float32),
#                                      ('z', np.float32),
#                                      ('rgb', np.float32)])).squeeze()
#     # pc_data = np.rec.fromarrays([xyz[:,0], xyz[:,1], xyz[:,2]], dtype=dt)
#     # data = np.rec.fromarrays([xyz.T], dtype=dt)
#     pc = PointCloud(md, pc_data)
#     return pc
#
#
# def encode_rgb_for_pcl(rgb):
#     """ Encode bit-packed RGB for use with PCL.
#     :param rgb: Nx3 uint8 array with RGB values.
#     :rtype: Nx1 float32 array with bit-packed RGB, for PCL.
#     """
#     assert(rgb.dtype == np.uint8)
#     assert(rgb.ndim == 2)
#     assert(rgb.shape[1] == 3)
#     rgb = rgb.astype(np.uint32)
#     rgb = np.array((rgb[:, 0] << 16) | (rgb[:, 1] << 8) | (rgb[:, 2] << 0),
#                    dtype=np.uint32)
#     rgb.dtype = np.float32
#     return rgb
#
#
# def decode_rgb_from_pcl(rgb):
#     """ Decode the bit-packed RGBs used by PCL.
#     :param rgb: An Nx1 array.
#     :rtype: Nx3 uint8 array with one column per color.
#     """
#
#     rgb = rgb.copy()
#     rgb.dtype = np.uint32
#     r = np.asarray((rgb >> 16) & 255, dtype=np.uint8)
#     g = np.asarray((rgb >> 8) & 255, dtype=np.uint8)
#     b = np.asarray(rgb & 255, dtype=np.uint8)
#     rgb_arr = np.zeros((len(rgb), 3), dtype=np.uint8)
#     rgb_arr[:, 0] = r
#     rgb_arr[:, 1] = g
#     rgb_arr[:, 2] = b
#     return rgb_arr
#
#
# def make_xyz_label_point_cloud(xyzl, label_type='f'):
#     """ Make XYZL point cloud from numpy array.
#     TODO i labels?
#     """
#     md = {'version': .7,
#           'fields': ['x', 'y', 'z', 'label'],
#           'count': [1, 1, 1, 1],
#           'width': len(xyzl),
#           'height': 1,
#           'viewpoint': [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
#           'points': len(xyzl),
#           'data': 'ASCII'}
#     if label_type.lower() == 'f':
#         md['size'] = [4, 4, 4, 4]
#         md['type'] = ['F', 'F', 'F', 'F']
#     elif label_type.lower() == 'u':
#         md['size'] = [4, 4, 4, 1]
#         md['type'] = ['F', 'F', 'F', 'U']
#     else:
#         raise ValueError('label type must be F or U')
#     # TODO use .view()
#     xyzl = xyzl.astype(np.float32)
#     dt = np.dtype([('x', np.float32), ('y', np.float32), ('z', np.float32),
#                    ('label', np.float32)])
#     pc_data = np.rec.fromarrays([xyzl[:, 0], xyzl[:, 1], xyzl[:, 2],
#                                  xyzl[:, 3]], dtype=dt)
#     pc = PointCloud(md, pc_data)
#     return pc
#
#
class PointCloud(object):
    """ Wrapper for point cloud data.
    The variable members of this class parallel the ones used by
    the PCD metadata (and similar to PCL and ROS PointCloud2 messages),
    ``pc_data`` holds the actual data as a structured numpy array.
    The other relevant metadata variables are:
    - ``version``: Version, usually .7
    - ``fields``: Field names, e.g. ``['x', 'y' 'z']``.
    - ``size.`: Field sizes in bytes, e.g. ``[4, 4, 4]``.
    - ``count``: Counts per field e.g. ``[1, 1, 1]``. NB: Multi-count field
      support is sketchy.
    - ``width``: Number of points, for unstructured point clouds (assumed by
      most operations).
    - ``height``: 1 for unstructured point clouds (again, what we assume most
      of the time.
    - ``viewpoint``: A pose for the viewpoint of the cloud, as
      x y z qw qx qy qz, e.g. ``[0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0]``.
    - ``points``: Number of points.
    - ``type``: Data type of each field, e.g. ``[F, F, F]``.
    - ``data``: Data storage format. One of ``ascii``, ``binary`` or ``binary_compressed``.
    See `PCL docs <http://pointclouds.org/documentation/tutorials/pcd_file_format.php>`__
    for more information.
    """

    def __init__(self, metadata, pc_data):
        self.metadata_keys = metadata.keys()
        self.__dict__.update(metadata)
        self.pc_data = pc_data
        self.check_sanity()

    def get_metadata(self):
        """ returns copy of metadata """
        metadata = {}
        for k in self.metadata_keys:
            metadata[k] = copy.copy(getattr(self, k))
        return metadata

    def check_sanity(self):
        # pdb.set_trace()
        md = self.get_metadata()
        assert(_metadata_is_consistent(md))
        assert(len(self.pc_data) == self.points), "{}, {}".format(len(self.pc_data), self.points)
        assert(self.width*self.height == self.points)
        assert(len(self.fields) == len(self.count))
        assert(len(self.fields) == len(self.type))

#     def save(self, fname):
#         self.save_pcd(fname, 'ascii')
#
#     def save_pcd(self, fname, compression=None, **kwargs):
#         if 'data_compression' in kwargs:
#             warnings.warn('data_compression keyword is deprecated for'
#                           ' compression')
#             compression = kwargs['data_compression']
#         with open(fname, 'w') as f:
#             point_cloud_to_fileobj(self, f, compression)
#
#     def save_pcd_to_fileobj(self, fileobj, compression=None, **kwargs):
#         if 'data_compression' in kwargs:
#             warnings.warn('data_compression keyword is deprecated for'
#                           ' compression')
#             compression = kwargs['data_compression']
#         point_cloud_to_fileobj(self, fileobj, compression)
#
#     def save_pcd_to_buffer(self, compression=None, **kwargs):
#         if 'data_compression' in kwargs:
#             warnings.warn('data_compression keyword is deprecated for'
#                           ' compression')
#             compression = kwargs['data_compression']
#         return point_cloud_to_buffer(self, compression)
#
#     def save_txt(self, fname):
#         save_txt(self, fname)
#
#     def save_xyz_label(self, fname, **kwargs):
#         save_xyz_label(self, fname, **kwargs)
#
#     def save_xyz_intensity_label(self, fname, **kwargs):
#         save_xyz_intensity_label(self, fname, **kwargs)
#
#     def copy(self):
#         new_pc_data = np.copy(self.pc_data)
#         new_metadata = self.get_metadata()
#         return PointCloud(new_metadata, new_pc_data)
#
#     def to_msg(self):
#         if not HAS_SENSOR_MSGS:
#             raise Exception('ROS sensor_msgs not found')
#         # TODO is there some metadata we want to attach?
#         return numpy_pc2.array_to_pointcloud2(self.pc_data)
#
    @staticmethod
    def from_path(fname):
        return point_cloud_from_path(fname)
#
#     @staticmethod
#     def from_fileobj(fileobj):
#         return point_cloud_from_fileobj(fileobj)
#
    @staticmethod
    def from_buffer(buf):
        return point_cloud_from_buffer(buf)
#
#     @staticmethod
#     def from_array(arr):
#         """ create a PointCloud object from an array.
#         """
#         pc_data = arr.copy()
#         md = {'version': .7,
#               'fields': [],
#               'size': [],
#               'count': [],
#               'width': 0,
#               'height': 1,
#               'viewpoint': [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
#               'points': 0,
#               'type': [],
#               'data': 'binary_compressed'}
#         md['fields'] = pc_data.dtype.names
#         for field in md['fields']:
#             type_, size_ =\
#                 numpy_type_to_pcd_type[pc_data.dtype.fields[field][0]]
#             md['type'].append(type_)
#             md['size'].append(size_)
#             # TODO handle multicount
#             md['count'].append(1)
#         md['width'] = len(pc_data)
#         md['points'] = len(pc_data)
#         pc = PointCloud(md, pc_data)
#         return pc
#
#     @staticmethod
#     def from_msg(msg, squeeze=True):
#         """ from pointcloud2 msg
#         squeeze: fix when clouds get 1 as first dim
#         """
#         if not HAS_SENSOR_MSGS:
#             raise NotImplementedError('ROS sensor_msgs not found')
#         md = {'version': .7,
#               'fields': [],
#               'size': [],
#               'count': [],
#               'width': msg.width,
#               'height': msg.height,
#               'viewpoint': [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
#               'points': 0,
#               'type': [],
#               'data': 'binary_compressed'}
#         for field in msg.fields:
#             md['fields'].append(field.name)
#             t, s = pc2_type_to_pcd_type[field.datatype]
#             md['type'].append(t)
#             md['size'].append(s)
#             # TODO handle multicount correctly
#             if field.count > 1:
#                 warnings.warn('fields with count > 1 are not well tested')
#             md['count'].append(field.count)
#         pc_array = numpy_pc2.pointcloud2_to_array(msg)
#         pc_data = pc_array.reshape(-1)
#         md['height'], md['width'] = pc_array.shape
#         md['points'] = len(pc_data)
#         pc = PointCloud(md, pc_data)
#         return pc


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-05-30 13:08
Short Description:

Change History:

'''

import requests
from requests.adapters import HTTPAdapter
import inspect
import json
from peutils.textutil import gen_uuid
from typing import Dict, List,Union


class DotDict(dict):
    def __init__(self,*args,**kwargs):
        super(DotDict, self).__init__(*args,**kwargs)

    def __getattr__(self, key):
        value = self[key]
        if isinstance(value,dict):
            value = DotDict(value)
        return value

# print(json.dumps(DotDict()))
# print(json.dumps(a))
# print(a["a"])
# a = DotDict()
# a["a"] = 1
#
# print(a)
# print(bool(DotDict()))

def get_session(retry=3):
    session = requests.Session()
    session.mount('http://', HTTPAdapter(max_retries=retry))
    session.mount('https://', HTTPAdapter(max_retries=retry))

    return session

# print(inspect.isbuiltin(int))

def dict_adapter(d:dict,out_adapter=None, rename:dict=None):
    d = {k: v for k, v in d.items()}
    if out_adapter is not None:
        if inspect.isfunction(out_adapter):
            for k,v in d.items():
                d[k] = out_adapter(v)
        elif isinstance(out_adapter,dict):
            for key, out_func in out_adapter.items():
                d[key] = out_func(d[key])
        else:
            raise Exception("参数必须是函数或者字典. 如果是字典，那么就是对应的值是方法")

    if rename is not None:
        ## 重写一遍所有的，但是这边不能有原来存在过的键，否则会冲突
        if len(set(rename.values()) & set(d.keys())) > 0:
            raise Exception(f"新的名称不能存在之前的名称中 {repr(set(rename.values()) & set(d.keys()))}")
        else:
            d = {rename.get(k, k): v for k, v in d.items()}
    return d


### 不兼容老的Plss模版，只兼容新的模版.
### 都存在的 id, msg, category(可选). number(可选). frameNum(可选，如果单帧就是0)
class ErrorUnit():
    def __init__(self,id,message,category=None,number=None,frameNum:Union[List[int],int,None]=None,block=True):
        self.id = id
        self.message = message
        self.category = category
        self.number = number
        self.frameList = self.get_frameList(frameNum)
        self.bock = block

    def get_frameList(self,frameNum):
        if frameNum is None:
            return [0]
        elif isinstance(frameNum,int):
            return [frameNum]
        elif isinstance(frameNum,list):
            return list(frameNum)
        else:
            raise Exception(f"不正确的frameNum定义 {frameNum}")

    def __repr__(self):
        if self.frameList !=[0]:
            err_str = f"帧:{[x+1 for x in self.frameList]} ID:{self.id} Message:{self.message} "
        else:
            err_str = f"ID:{self.id} Message:{self.message} "

        if self.category!="":
            err_str += f"物体:{self.category} {self.number}"

        return err_str

'''
平台错误的格式
'''

class ErrorMsgLogV1():

    def __init__(self):
        self.error_list = []

    def create_error(self,msg,obj=None,frameNum:Union[List[int],int,None]=None,block=True):
        if obj is None:
            self.error_list.append(ErrorUnit(
                id = "common-" + gen_uuid(),
                message= msg,
                category="",
                number="",
                frameNum=frameNum,
                block=block
            ))
        else:
            self.error_list.append(ErrorUnit(
                id = obj.id,
                message = msg,
                category = obj.category,
                number = obj.number,
                frameNum=obj.frameNum,
                block=block
            ))


    def fomart_error_str(self)->str:
        return "\n".join([repr(e) for e in self.error_list])
        # 如果frame是0，不打印。

    def format_a9_error_str(self)->str:
        # 平台只接受id,message,frames,block四个属性。
        return json.dumps([{
            "id": e.id,
            "message": e.message,
            "frames":e.frameList,
            "block":e.block
        } for e in self.error_list],ensure_ascii=False)




class Lidar3dObj():
    def __init__(self, frameNum, id, number, category, position, rotation, dimension,
                 lidar_attr=None, quaternion=None,pointCount=None):
        self.frameNum = frameNum
        self.id = id
        self.number = number
        self.category = category
        self.position = DotDict(position)
        self.rotation = DotDict(rotation)
        self.dimension = DotDict(dimension)

        self.lidar_attr = DotDict(lidar_attr) if lidar_attr else DotDict() # 属性
        self.quaternion = DotDict(quaternion) if quaternion else DotDict()
        self.pointCount = pointCount

    def __repr__(self):
        return f"{self.id} {self.category} {self.number}"

    def to_dict(self):
        _data_dict = {
            "frameNum":self.frameNum,
            "id":self.id,
            "number":self.number,
            "category":self.category,
            "position":self.position,
            "rotation":self.rotation,
            "dimension":self.dimension,
            "labels":"" if self.lidar_attr else json.dumps(self.lidar_attr,ensure_ascii=False),
        }
        return _data_dict


class Lidar3dImageRect():
    def __init__(self, frameNum, id, number,type, category, position, dimension,imageNum=None,
                 img_attr=None):
        self.frameNum = frameNum
        self.imageNum = imageNum # 图像的次序。从0开始
        self.id = id
        self.number = number
        self.type = type
        self.category = category
        self.position = DotDict(position)
        self.dimension = DotDict(dimension)

        self.img_attr = DotDict(img_attr) if img_attr else DotDict()  # 属性

        self.bbox =self.get_bbox()


    def get_bbox(self):
        # xmin ymin w h
        return [
            self.position["x"],self.position["y"],
            self.dimension["x"],self.dimension["y"]
        ]

    def to_dict(self):
        _data_dict = {
            "type":self.type,
            "id":self.id,
            "number":self.number,
            "category":self.category,
            "position":self.position,
            "dimension":self.dimension,
            "labels":"" if self.img_attr else json.dumps(self.img_attr,ensure_ascii=False),
        }
        return _data_dict

    def __repr__(self):
        return f"{self.id} {self.category} {self.number} {self.imageNum}"


'''
categoryColor一般为空不要用，
'''
class ImgInstance():
    def __init__(self,id,category,number,categoryName=None,ist_attr=None):
        # self.frameNum = frameNum  # frameNum用子物体的
        self.id = id
        self.category = category
        self.categoryName = categoryName
        self.number = number
        # self.categoryColor = categoryColor # 不用这个属性
        self.ist_attr = DotDict(ist_attr) if ist_attr else DotDict()
        self.obj_list = []

    def __repr__(self):
        return f"{self.id} {self.category} {self.number} {len(self.obj_list)}T"

    def to_pre_dict(self):
        _pre_data_dict = {
            "id":self.id,
            "category":self.category,
            "number": self.number,
            "attributes":self.ist_attr,
            "children":[
            ]
        }

        ## 计算child_dict
        child_dict = dict()
        for item in self.obj_list:
            child_id = item.id
            if child_id not in child_dict:
                child_dict[child_id] = {
                    "id": item.id,
                    "name": item.category,
                    "number": item.number,
                    "cameras": [{
                        "camera": "default",
                        "frames": []
                    }]
                }
            ## 添加这条数据到frames信息中
            child_dict[child_id]["cameras"][0]["frames"].append({
                "frameIndex": item.frameNum,
                "isKeyFrame": True,
                "shapeType": item.shapeType,
                "shape": item.shape,
                "order": item.order,
                "attributes": item.img_attr
            })
        for _,v in child_dict.items():
            _pre_data_dict["children"].append(v)
        return  _pre_data_dict




class Img2Dobj():
    def __init__(self,instance:ImgInstance,
                 frameNum,id,number,category,
                 shapeType,order=None,shape=None,img_attr=None,
                 displayName="", color="",
                 isOCR=None,OCRText=None
                 ):
        self.instance = instance
        self.frameNum = frameNum
        self.id = id
        self.category = category
        self.displayName = displayName
        self.number = number
        self.color = color
        self.shapeType = shapeType
        self.shape = DotDict(shape) if shape else DotDict()
        self.order = order
        self.img_attr = DotDict(img_attr) if img_attr else DotDict()
        self.isOCR = isOCR
        self.OCRText = OCRText

    def __repr__(self):
        return f"F{self.frameNum} {self.id} {self.category} {self.number} {self.shapeType} Order:{self.order} <-[{self.instance.category}]"


class AudioCutObj():
    def __init__(self,frameNum,id,number,start,end,block_attr,line_contents,category=""):
        self.frameNum = frameNum
        self.id = id
        self.number = number
        self.category = category
        self.start = start
        self.end = end
        self.block_attr = DotDict(block_attr) if block_attr else DotDict()
        self.line_contents = line_contents # 对应content字段

    def __repr__(self):
        return f"{self.id} {self.category} {self.number} {self.start} {self.end}"


# type points 点,polyline 线
# pointCount 最好根据数组长度重新算下
# 点云分割只有实例模式
'''
pointsLabels
[ 
    [
       12.747154235839844,
       5.126907825469971,
       -1.3931894302368164,
       86
    ],
    ...
]
'''

class LidarPointObj():
    def __init__(self,frameNum ,id,category,number,points,point_attr,pointsLabels,pointCount,type):
        self.frameNum = frameNum
        self.id = id
        self.category = category
        self.number = number
        self.points = points
        self.point_attr = point_attr
        self.pointsLabels = pointsLabels # 这个是点的 坐标和反射率信息的数组
        self.pointCount = pointCount
        self.type = type  # 平台工具多变形，笔刷，单点，都是point,折线是polyline

    def __repr__(self):
        return f"{self.id} {self.category} {self.number} {len(self.points)}P"

class LidarPointPolyline():
    def __init__(self, frameNum, imageNum, id, number, type, category, points,
                 img_attr=None):
        self.frameNum = frameNum
        self.imageNum = imageNum  # 图像的次序。从0开始
        self.id = id
        self.number = number
        self.type = type
        self.category = category
        self.points = points
        self.img_attr = img_attr  # 属性


    def to_dict(self):
        _data_dict = {
            "type": self.type,
            "id": self.id,
            "number": self.number,
            "category": self.category,
            "points": self.points,
            "labels": "" if self.img_attr is None else json.dumps(self.img_attr, ensure_ascii=False),
        }
        return _data_dict

    def __repr__(self):
        return f"{self.id} {self.category} {self.number} {self.imageNum}"


class CommonBaseMixIn():
    session = get_session(3)

    def get_raw_data(self,url):
        rs =  self.session.get(url).json()
        return rs


import math
def gen_format_progress_seq(total,split_part=10):
    ###初始化
    total = total # 总的数量
    split_part = split_part # 分片数量
    every_part_num = math.ceil(total/split_part) # 总打印的分片数量

    finish = 0  # 当前完成的数量
    finish_part = 0 # 分配开始的进度数量
    # 分片数量，假如 总量是102，分片是10
    def update(step=1):
        nonlocal finish,finish_part
        finish += step # 每调用一次 加1
        if finish//every_part_num >finish_part and finish//every_part_num <=split_part:
            finish_part+=1 # 分片数量加 1
            print( "[",("*"*finish_part).ljust(split_part,"_") ,"]" )
        # 根据完成数量update进度
    return update

import time
def deco_execution_time(func):
    def wrapper(*args, **kw):
        t_begin = time.time()
        res = func(*args, **kw)
        t_end = time.time()

        if t_end - t_begin < 60:
            print('%s executed in %s (s)' % (func.__name__, round(t_end - t_begin,2)) )
        elif t_end -t_begin <3600:
            print('%s executed in %s (min)' % (func.__name__, round((t_end - t_begin)/60, 2)))
        else:
            print('%s executed in %s (h)' % (func.__name__, round((t_end - t_begin) /3600, 2)))

        return res

    return wrapper

def remove_key_if_exists(info_dict: dict, rm_list: list):
    for rm_name in rm_list:
        if rm_name in info_dict:
            del info_dict[rm_name]

    return info_dict


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-05-30 13:08
Short Description:

Change History:

'''

from peutils.transform.v1.base import *
import json
from types import SimpleNamespace
from collections import defaultdict
from peutils.datautil import gen_uuid_seq



class LidarBoxFrame():
    def __init__(self,frameId,frameUrl,isValid,frameUrlInternal,frameUrlExternal,
                 frame_attr,items,images,
                 config):
        self.frameId = frameId
        self.frameUrl = frameUrl
        self.isValid = isValid
        self.frameUrlInternal = frameUrlInternal
        self.frameUrlExternal = frameUrlExternal

        self.log = ErrorMsgLogV1()

        self.config = config
        if self.config.parse_id_col =="fid":
            self.config.seq_func = gen_uuid_seq(start=self.config.seq_start)

        self.frame_attr = frame_attr
        self.lidar_dict = self.get_lidar_dict(items)
        self.camera_list,self.camera_meta,self.images_dict,self.images_list= self.get_images_dict(images)
        self._img_idset = self.get_img_idset(self.images_dict)
        self.only_lidar_idset= self.lidar_dict.keys() - self._img_idset # 只有3D 没有出现在2D的ID
        self.only_image_idset = self._img_idset -  self.lidar_dict.keys() # 只出现在2D没有出现在3D中的ID
        self.has_23d_idset = self.lidar_dict.keys() & self._img_idset # 2D和3D都出现


    @staticmethod
    def get_img_idset(images_dict):
        img_idset = set()
        for _,v in images_dict.items():
            img_idset = img_idset | v.keys()
        return img_idset

    def get_img_obj_list(self,id,keep_nan=False):
        lst = []
        for cam in self.camera_list:
            obj = self.images_dict[cam].get(id)
            if keep_nan == True:
                lst.append(obj)
            else:
                if obj:
                    lst.append(obj)
        return lst


    def parse_lidar_by_item(self,item):

        rotation =  dict_adapter(item["rotation"],out_adapter=self.config.number_adpter_func)
        position = dict_adapter(item["position"],out_adapter=self.config.number_adpter_func)
        dimension = dict_adapter(item["dimension"],out_adapter=self.config.number_adpter_func)
        ## 四元数
        quaternion =dict_adapter(item["quaternion"],out_adapter=self.config.number_adpter_func)

        ### 解析点云数量
        if item.get("pointCount") and item.get("pointCount").get("lidar") is not None:
            pointCount = item["pointCount"]["lidar"]
        else:
            pointCount = None


        '''
        检查逻辑
        1.如果yaw_only True,检查 rotation x 和 y都是0
        2.如果has_pointCount是True,检查pointCount 不是None
        '''
        lidar_obj = Lidar3dObj(
            frameNum= self.frameId, #item["frameNum"],
            id = item["id"],
            number= item["number"],
            category=item["category"],
            position= position,
            rotation= rotation,
            dimension = dimension,
            quaternion = quaternion,
            lidar_attr = json.loads(item["labels"]) if item.get("labels") else dict(),
            pointCount = pointCount
        )

        if self.config.parse_id_col =="id":
            key = item["id"]
        elif self.config.parse_id_col =="number":
            key = item["number"]
        elif self.config.parse_id_col in ("gid","fid"):
            key = self.config.seq_func(item["id"])
        else:
            raise Exception("parse_id_col解析模式错误")

        ####业务检查
        if self.config.has_pointCount == True and lidar_obj.pointCount is None:
            self.log.create_error(msg="点云数量不能为空",obj=lidar_obj)

        if self.config.yaw_only ==True:
            if lidar_obj.rotation["x"] !=0 or lidar_obj.rotation["y"] !=0:
                self.log.create_error(msg="仅水平旋转x y应该为0",obj=lidar_obj)


        return key,lidar_obj

    def parse_img_by_item(self,item,width,height):

        position = dict_adapter(item["position"], out_adapter=self.config.number_adpter_func)
        dimension = dict_adapter(item["dimension"], out_adapter=self.config.number_adpter_func)

        '''
        检查逻辑
        1.图像是否超出边界外

        '''
        if item["type"] =="RECT":
            img_obj = Lidar3dImageRect(
                frameNum= self.frameId, #item["frameNum"],
                imageNum=item["imageNum"],
                id=item["id"],
                number=item["number"],
                type=item["type"],
                category=item["category"],
                position=position,
                dimension=dimension,
                img_attr=json.loads(item["labels"]) if item.get("labels") else dict(),
            )
        else:
            # 暂时不会遇到
            img_obj =None

        if self.config.parse_id_col == "id":
            key = item["id"]
        elif self.config.parse_id_col == "number":
            key = item["number"]
        elif self.config.parse_id_col in ("gid","fid"):
            key = self.config.seq_func(item["id"])
        else:
            raise Exception("parse_id_col解析模式错误")

        if self.config.overflow ==False:
            if img_obj.position["x"] <0 or img_obj.position["y"]<0 \
                    or (img_obj.position["x"] + img_obj.dimension["x"]) > width \
                    or (img_obj.position["y"] + img_obj.dimension["y"]) >height:
                self.log.create_error(msg="图像超出边界",obj=img_obj)

        return key, img_obj


    def get_lidar_dict(self,items):
        lidar_dict = dict()

        for item in items:
            key,lidar_obj =self.parse_lidar_by_item(item=item)
            lidar_dict[key] = lidar_obj

        if len(items) != len(lidar_dict):
            raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
        return lidar_dict

    def get_single_image_dict(self,items,width,height):
        single_image_dict = dict()

        for item in items:
            if item["type"] =="RECT":
                key,img_obj = self.parse_img_by_item(item,width,height)
                single_image_dict[key] = img_obj
            else:
                raise Exception("目前图像物体仅支持RECT,其他类型还在开发中")

        if len(items) != len(single_image_dict):
            raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
        return single_image_dict

    def get_images_dict(self,images):
        ### 先解析出镜头数量
        images_dict = dict() #{cam:dict1,cam2:dict2,...}
        images_list = [] #[dict1,dict2,..]
        camera_list = []
        camera_meta = dict()

        for img in images:
            image_path = img["image"]
            camera_name = image_path.split("/")[-2]
            camera_list.append(camera_name)

            camera_meta[camera_name]= {
                "image_path":image_path,
                "imageUrlInternal":img["imageUrlInternal"],
                "imageUrlExternal":img["imageUrlExternal"],
                "width": img["width"],
                "height": img["height"],

            }
            sg_img_dict = self.get_single_image_dict(img["items"],width=img["width"],height = img["height"])
            images_dict[camera_name] = sg_img_dict
            images_list.append(sg_img_dict)

        ## check
        if len(camera_list) != len(images):
            raise Exception("找到的镜头数量和原始不一致")

        return camera_list,camera_meta,images_dict,images_list


    def __repr__(self):
        return f'Frame {self.frameId}'




class LidarBoxParse(CommonBaseMixIn):

    ### 继承session属性 用来读取url
    def __init__(self,url,config):
        self.url = url
        self.config = config
        if self.config.parse_id_col == "gid":
            self.config.seq_func = gen_uuid_seq(start=self.config.seq_start)

        self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
        self.frames_lst, self.frame_length = self.parse_by_frame()


    def check_frames_error(self):
        all_errors = []
        for frame in self.frames_lst:
            all_errors.extend(frame.log.error_list)
        return all_errors


    def parse_by_frame(self):
        '''
        返回解析后的frame列表，对应的数据
        '''
        #
        frames_lst = []
        for raw_frame in self.raw_data["frames"]:

            ### 属性
            attribute = raw_frame.get("attribute")
            if attribute:
                frame_attr = json.loads(attribute)
            else:
                frame_attr = dict()

            frame = LidarBoxFrame(
                frameId=raw_frame["frameId"],
                frameUrl=raw_frame["frameUrl"],
                isValid=raw_frame["isValid"],
                frameUrlInternal=raw_frame["frameUrlInternal"],
                frameUrlExternal=raw_frame["frameUrlExternal"],
                frame_attr = frame_attr,
                items = raw_frame["items"],
                images = raw_frame["images"],
                config= self.config
            )
            frames_lst.append(frame)
        return frames_lst,len(frames_lst)



class LidarBoxDataConfig():
    def __init__(self,yaw_only=True,has_pointCount=True,number_adpter_func=None,
                 parse_id_col="id",seq_start=0,overflow=False):
        self.yaw_only = yaw_only
        self.has_pointCount = has_pointCount
        self.number_adpter_func = number_adpter_func
        self.parse_id_col = parse_id_col # id或number ,gid, fid
        self.seq_start = seq_start
        self.overflow = overflow # 默认不允许超出图像边界

from pprint import pprint

if __name__ =="__main__":
    ## 单帧
    from pprint import pprint
    lidar = LidarBoxParse(url="https://oss-prd.appen.com.cn:9001/tool-prod/6e3149fd2e517f8ead3102ef2b23a55d/R.1654671141185.762645cb-d524-42b8-b5c2-217c002b2066.CLuksAIQAA3d3d_2022-06-08T065143Z.18179.QA_RW.4a1fb325-19b0-4dc4-92d4-c4a95eb29831.38888b56a8a16cda2f27473749a89622.review.json",
                         config =LidarBoxDataConfig(
                             yaw_only = True,
                             has_pointCount= True,
                             number_adpter_func=None, #lambda i: round(i,3), # 默认None
                             parse_id_col = "gid",
                             seq_start = 0, # 如果是 gid,fid,或者frame_id 需要提供seq_start， 如果是0就是从1开始编号，如果是-1就是从0开始编号
                             overflow= False
                         ))
    # pprint(lidar.check_frames_error())
    # pprint(lidar.frames_lst[0].lidar_dict)
    # for k,v in lidar.frames_lst[0].lidar_dict.items():
    #     print(k,v.position,v.pointCount,v.lidar_attr)
    # pprint(lidar.frames_lst[1].lidar_dict)
    # pprint(lidar.frames_lst[0].lidar_dict["06e41560-32ac-436d-a0c0-3e4aae7d4858"].rotation)
    # print(lidar.check_frames_error())
    # print(lidar.frames_lst[7].log.fomart_error_str())
    # print(lidar.check_frames_error())
    # print(json.dumps(lidar()))
    # p = lidar.frames_lst[0].lidar_dict["06e41560-32ac-436d-a0c0-3e4aae7d4858"].position
    # print(dict_adapter(p,rename={"x":"k","y":"x"},out_adapter=None))
    # {'x': 18.690793403437375, 'y': -56.18997617593776, 'z': -1.380000000000006}

    # print(lidar.frames_lst[0].camera_list)
    # print(lidar.frames_lst[0].camera_meta)
    # pprint(lidar.frames_lst[0].images_dict)
    # pprint(lidar.frames_lst[0].images_list)
    # print(lidar.frames_lst[0].only_image_idset)
    # print(lidar.frames_lst[0].get_img_obj_list(id="f661d8a6-ad24-4f95-b034-67983b17709f"))
    # print(lidar.frames_lst[0].get_img_obj_list(id="f661d8a6-ad24-4f95-b034-67983b17709f",keep_nan=True))







# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-14 14:33
Short Description:

Change History:

'''
from peutils.datautil import GenCategorySeq
from peutils.textutil import gen_uuid
import json
class AudioSegPre():
    def __init__(self,frame_length):
        self.audios_lst = [
            {"attributes":dict()} for i in range(frame_length)
        ]
        self.results = [
            [

            ] for i in range(frame_length)
        ]
        # self.frameorder_seq = GenCategorySeq()

    def add_audio_cut(self,frameNum,start,end,text=None,line_attr=None,block_attr=None,line_contents=None):
        ### 如果提供line_contents，那么就不用 text，line_atrr
        '''
        "content":[
                    {
                        "role":"none",
                        "text":"",
                        "attributes":{
                        }
                    }
                ]
        '''
        ###
        if line_contents is not None:
            if text is not None or line_attr is not None:
                raise Exception("提供了line_contents的情况下不需要再提供text和line_attr")
        else: # 为空的时候构建一个单行的
            line_contents = [{
                            "role":"none",
                            "text":text,
                            "attributes":line_attr if line_attr else dict()
                        }]
        self.results[frameNum].append(
            {
                "id":gen_uuid(),
                "start":start,
                "end":end,
                "attributes":block_attr if block_attr else dict(),
                "content":line_contents
            }
        )


    def dumps_data(self):
        _data = json.dumps({
            "results":self.results,
            "audios":self.audios_lst,
        },ensure_ascii=False)
        return _data


if __name__ =="__main__":
    adpre = AudioSegPre(frame_length=1)
    adpre.add_audio_cut(frameNum=0,start=1,end=2,text="abc")
    print(adpre.dumps_data())





# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-01-19 13:58
Short Description:

Change History:

'''

import smtplib
import email.utils
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText





class SendEmail:
    def __init__(self,name,pwd):
        self.name = name
        self.pwd = pwd


    def send_email(self, receiver, cc, content, subject=''):
        SENDER = self.name# 邮箱名
        SENDERNAME ="APPEN CS"
        msg = MIMEMultipart('related')
        msg['Subject'] = subject
        msg['From'] = email.utils.formataddr((SENDERNAME, SENDER))
        msg['to'] = ",".join(receiver)
        msg['Cc'] = ",".join(cc)
        USERNAME_SMTP = self.name # 带有邮件权限的 IAM 帐号
        PASSWORD_SMTP = self.pwd# 带有邮件权限的 IAM 密码
        HOST = "smtpdm.aliyun.com"
        PORT = 80

        thebody = MIMEText(content, 'html', 'utf-8')
        msg.attach(thebody)

        try:
            server = smtplib.SMTP(HOST, PORT)
            #server.ehlo()
            server.starttls()
            #server.ehlo()
            server.login(USERNAME_SMTP, PASSWORD_SMTP)
            server.sendmail(SENDER, receiver + cc, msg.as_string())
            server.close()
        except Exception as e:
            print("Error: ", e)


s = SendEmail(name="xxx",pwd="xxx")
s.send_email(receiver = ["hwang2@appen.com","yazhang@appen.com"],cc=[],subject="邮件标题",content="邮件内容")



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-09 14:51
Short Description:

Change History:

'''
from peutils.transform.v1.base import *
import json
from peutils.datautil import gen_uuid_seq


class ImgComFrame():
    def __init__(self,frameId,frameUrl,isValid,imageWidth,imageHeight,
                 rotation,
                 frame_attr,
                 config):
        self.frameId = frameId
        self.frameUrl = frameUrl
        self.isValid = isValid
        self.imageWidth = imageWidth
        self.imageHeight = imageHeight
        self.rotation = rotation

        self.log = ErrorMsgLogV1()
        self.config = config
        self.frame_attr = frame_attr
        self.frame_items = []


    def add_frame_obj(self,obj:Img2Dobj):
        if obj.order is None:
            self.log.create_error("order检查模式下不能缺失",obj=obj)
        self.frame_items.append(obj)

    # def to_pre_dict(self):
    #     _pre_dict = {
    #         "frameId":self.frameId,
    #         "frame_attr":self.frame_attr,
    #         "frame_items":self.frame_items
    #     }
    #     return _pre_dict


    def __repr__(self):
        return f'Frame {self.frameId} {len(self.frame_items)}T'


class ImgComParse(CommonBaseMixIn):

    ### 继承session属性 用来读取url
    def __init__(self,url,config):
        self.url = url
        self.config = config
        self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
        self.instance_lst, self.object_lst = self.parse_by_instance()
        self.frames_lst, self.frame_length = self.parse_by_frame()


    def check_frames_error(self):
        all_errors = []
        for frame in self.frames_lst:
            all_errors.extend(frame.log.error_list)
        return all_errors

    def parse_by_frame(self):
        frames_lst = []
        for raw_frame in self.raw_data["frames"][0]["frames"]: # 先解析default cam
            frame = ImgComFrame(
                frameId= raw_frame["frameIndex"],
                frameUrl= raw_frame["imageUrl"],
                imageWidth = raw_frame["imageWidth"],
                imageHeight = raw_frame["imageHeight"],
                isValid= raw_frame["valid"],
                rotation = raw_frame["rotation"],
                frame_attr= raw_frame.get("attributes",dict()),
                config=self.config
            )
            frames_lst.append(frame)

        for obj in self.object_lst:
            frames_lst[obj.frameNum].add_frame_obj(obj)

        ### 对order进行检查并且排序数据,缺失的数据排在最前面.
        for f in frames_lst:
            f.frame_items.sort(key=lambda i: -1 if i.order is None else i.order)

        return frames_lst,len(frames_lst)

    def parse_by_instance(self):

        instance_lst = []
        all_obj_lst = []
        for instance in self.raw_data["instances"]:
            ist = ImgInstance(
                id=instance["id"],
                category = instance["category"],
                categoryName = instance["categoryName"],
                # categoryColor = instance["categoryColor"],
                number = instance["number"],
                ist_attr = instance.get("attributes",dict())
            )
            obj_list = []
            for ch in instance["children"]:
                for obj in ch["cameras"][0]["frames"]: # 默认先只处理default cam
                    imgobj = Img2Dobj(
                        instance = ist,
                        frameNum= obj["frameIndex"],
                        id = ch["id"],
                        number = ch["number"],
                        category = ch["name"],
                        displayName = ch["displayName"],
                        color = ch["displayColor"],
                        shapeType = obj["shapeType"],
                        shape = obj["shape"],
                        order = obj.get("order"),# 测试None if ch["id"] =="faeb43ff-546b-4f80-b99b-8e4eeb62f112" else obj.get("order"),
                        img_attr = obj.get("attributes",dict()),
                        isOCR= obj.get("isOCR",False),
                        OCRText= obj.get("OCRText","")
                    )
                    obj_list.append(imgobj)
            # 加到ist实例中
            all_obj_lst.extend(obj_list)
            ist.obj_list.extend(obj_list)
            instance_lst.append(ist)

        return instance_lst,all_obj_lst



class ImgComDataConfig():
    def __init__(self,parse_id_col="id",number_adpter_func=None,seq_start=0,check_order=True,overflow=False):
        self.number_adpter_func = number_adpter_func
        self.parse_id_col = parse_id_col, # 默认id ## 暂时不引入fid,gid
        self.check_order = check_order # 检查order 是否都存在
        self.seq_start = seq_start  # 暂时用不到
        self.overflow = overflow # 默认不允许超出图像边界



if __name__ =="__main__":
    ## 单帧
    from pprint import pprint
    img = ImgComParse(url="https://oss-prd.appen.com.cn:9001/tool-prod/a2a3ef0c-55c4-4d15-8cc2-ff6aeb7878dd/R.1650783983510.a2a3ef0c-55c4-4d15-8cc2-ff6aeb7878dd.CODU4AEQEg3d3d_2022-04-24T070156Z.18107.result.json",
                         config =ImgComDataConfig(
                         ))
    # print(img.frames_lst[0].frame_items)

    # pprint(img.instance_lst)
    # pprint(img.instance_lst[0].obj_list)
    # pprint(img.object_lst)

    # pprint(img.frames_lst[0].frame_items)
    # for item in img.frames_lst[0].frame_items:
    #     print(item.instance)

    # pprint(img.frames_lst[0].)
    # img.frames_lst[9].frame_obj_list.sort(key=lambda i: i.order)
    # pprint(img.frames_lst[9].frame_items)
    # pprint(img.frames_lst[9].log.error_list)
    # pprint(img)
    # pprint(img.object_lst)

    # for frame in img.frames_lst:
    #     # print(frame.imageWidth)
    #     # print(frame.frame_obj_list)
    #     # print(frame.log.error_list)
    #     pprint(frame)
    # pprint(lidar.frames_lst[49].lidar_dict)


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-05 13:23
Short Description:

Change History:

'''
import os
from functools import partial
from pathlib import Path

'''
# Henry Wang, 2020-08-14 11:07 Changed:使用pathlib替换os.path 兼容windows
'''


def list_current_file(path='.', type='all', suffix='', not_prefix=(('~', '.'))):
    '''
    列出当前目录下的文件或者文件夹
    :param path: 默认当前目录
    :param type: 可选 file,folder,all 默认all
    :param suffix: 对文件夹和文件后缀过滤
    :param not_prefix: 对文件夹和文件前缀过滤，默认不要隐藏文件和临时文件
    :return:文件或文件夹的集合
    '''
    p = Path(path)

    if type == 'all':
        return [x.resolve().as_posix() for x in p.iterdir()
                if x.name.endswith(suffix) and not x.name.startswith(not_prefix)]
    elif type == "file":
        return [x.resolve().as_posix() for x in p.iterdir() if x.is_file() and  x.name.endswith(suffix) and not x.name.startswith(not_prefix)]
    elif type == "folder":
        return [x.resolve().as_posix() for x in p.iterdir() if x.is_dir() and  x.name.endswith(suffix) and not x.name.startswith(not_prefix)]
    else:
        raise Exception(f"type: {type} not defined.")


def list_files_deep(path='.', suffix='', not_prefix=(('~', '.'))):
    '''
    :param path: 默认当前目录 '.'
    :param suffix: 文件后缀，单个或者元组
    :param not_prefix: 单个或者元组,默认去掉隐藏文件和临时文件
    :return: 文件全路径集合
    '''
    files = []
    all_files = list(Path(path).glob('**/*.*'))  # 过滤出来的是文件

    # # Henry Wang, 2021-01-28 14:01 Changed:将suffix转成小写
    if isinstance(suffix,str)==True:
        suffix = suffix.lower()
    elif isinstance(suffix,tuple) ==True:
        suffix = tuple([x.lower() for x in suffix])

    # Henry Wang, 2021-05-25 14:18 Changed:增加os.path.isfile的判断
    for filpath in all_files:
        if filpath.is_file() ==True and filpath.name.lower().endswith(suffix) and not filpath.name.startswith(not_prefix):
            files.append(filpath.resolve().as_posix())

    return files


### 替换生成新地址，并创建文件夹
### 老的地址必须包含在
def replace_path_and_create(filepath, old_dir, new_dir, add_path: list):
    old_dir = Path(old_dir)
    new_dir = Path(new_dir)
    filepath = Path(filepath)

    if old_dir.resolve().as_posix() in filepath.resolve().as_posix():
        new_fl = filepath.resolve().as_posix().replace(old_dir.resolve().as_posix(), new_dir.resolve().as_posix())
        if add_path != None:
            p = Path(new_fl).resolve()
            new_fl = p.parent.joinpath(*add_path, p.name).resolve().as_posix()
        return new_fl
    else:
        raise Exception(f"{old_dir} not in {new_dir}")


'''
自定义方法，从A目录到B目录，找到对应后缀文件，循环并生成新的文件
'''


class Hanlde_Move_File():
    def __init__(self, origin_path, out_path, process_func, in_suffix, out_suffix, add_path=None):
        self.origin_path = origin_path
        self.out_path = out_path
        self.change_path = partial(replace_path_and_create, old_dir=origin_path, new_dir=out_path,
                                   add_path=add_path)  # 只要传filepath
        self.process_func = process_func
        self.in_suffix = in_suffix.strip()
        self.out_suffix = out_suffix.strip()
        self.add_path = add_path

    def main(self):
        print(f'handle {self.process_func.__name__.upper()} from: {self.origin_path} to: {self.out_path}')
        files = list_files_deep(self.origin_path, suffix=self.in_suffix)
        print(f'处理文件总数: {len(files)}')
        out_pre_file = list_files_deep(self.out_path, suffix=self.out_suffix)
        if len(out_pre_file) != 0:
            print(f'Warining: 目标文件夹，该类型文件存在 {out_pre_file} 个历史文件，请确认操作，无问题可忽略本提示')

        for filename in files:
            try:
                out_change_path = self.change_path(filename)[:-len(self.in_suffix)] + self.out_suffix
                out_path_parent = Path(out_change_path).resolve().parent
                out_path_parent.mkdir(parents=True, exist_ok=True)  # 递归创建目录，如果已存在不报错
                self.process_func(filename, out_change_path)

            except Exception as e:
                print(f'filename {filename} Error')

        out_file = list_files_deep(self.out_path, suffix=self.out_suffix)
        print(f"输出文件总数: {len(out_file)}")


'''
文件名相同的情况下检查文件对齐,不同后缀文件数量对齐,可以在相同文件夹，也可以在不同文件夹
'''



import csv


def getCsvData(csvPath):
    csv.field_size_limit(500 * 1024 * 1024)
    with open(csvPath, encoding='utf-8-sig') as f:
        data = csv.reader(f)
        header = next(data)
        data = list(data)
    return data, header


def saveCsvData(savePath, data: list, header=None, newline='\n'):
    csv.field_size_limit(500 * 1024 * 1024)
    with open(savePath, 'w', encoding='utf-8-sig', newline=newline) as f:
        writer = csv.writer(f, quoting=csv.QUOTE_ALL, lineterminator="\n")
        if header:
            writer.writerow(header)
        writer.writerows(data)

def getExcelData(excelPath,sheetname=None):
    from openpyxl import load_workbook
    wb = load_workbook(excelPath)
    if sheetname is None:
        ws  = wb.active
    else:
        ws = wb[sheetname]
    all_data =ws.rows
    header = [x.value for x in next(all_data)]
    data = [[x.value for x in row] for row in all_data]
    return data,header


def saveExcelData(savePath,data: list, header=None):
    from openpyxl import Workbook
    wb = Workbook()
    ws=wb.active
    if header is not None:
        ws.append(header)
    for row in data:
        ws.append(row)
    wb.save(savePath)



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-05 15:55
Short Description:

Change History:

'''
def gen_uuid():
    import uuid
    uid = str(uuid.uuid4())
    suid = ''.join(uid.split('-'))
    return suid


def md5_file(filename):
  '''
  GEN MD5
  WINDOWS CMD LINE :certutil -hashfile Filename MD5
  MAC/LINUX :md5sum Filename
  '''
  import hashlib
  md5_l = hashlib.md5()
  with open(filename,mode="rb") as f:
    data = f.read()
  md5_l.update(data)
  ret = md5_l.hexdigest()
  print(ret)


def gen_date_str(sep='std'):
    from datetime import datetime
    '''
    if sep is std return %Y-%m-%d %H:%M:%S  '2020-06-28 21:26:47'
    if sep is None return %Y%m%d%H%M%S '20200628212651'
    '''
    if sep == 'std':
        return datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    elif sep == None:
        return datetime.now().strftime("%Y%m%d%H%M%S")
    else:
        raise Exception('sep not definded.')


### 去掉前后空格，多个空格变成一个空格
def strip_and_replace_blank(text):
    import re
    newtext = re.sub(' +',' ',text).strip()
    return newtext


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-05 15:53
Short Description:

Change History:

'''

import logging
from logging import handlers


class Logger(object):
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'critical': logging.CRITICAL
    }  # 日志级别关系映射

    def __init__(self, filename, level='info', when='D', backCount=5,
                 fmt='%(asctime)s %(levelname)s || Message:%(message)s'):
        self.logger = logging.getLogger(filename)
        format_str = logging.Formatter(fmt)  # 设置日志格式
        self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
        # sh = logging.StreamHandler()#往屏幕上输出
        # sh.setFormatter(format_str) #设置屏幕上显示的格式
        th = handlers.TimedRotatingFileHandler(filename=filename, when=when, backupCount=backCount,
                                               encoding='utf-8')  # 往文件里写入#指定间隔时间自动生成文件的处理器
        # 实例化TimedRotatingFileHandler
        # interval是时间间隔，backupCount是备份文件的个数，如果超过这个个数，就会自动删除，when是间隔的时间单位，单位有以下几种：
        # S 秒
        # M 分
        # H 小时、
        # D 天、
        # W 每星期（interval==0时代表星期一）
        # midnight 每天凌晨
        th.setFormatter(format_str)  # 设置文件里写入的格式
        # self.logger.addHandler(sh) #把对象加到logger里
        self.logger.addHandler(th)


'''
# how to use logger

if __name__ == '__main__':
    infolog = Logger('all.log',level='debug').logger
    infolog.debug('debug')
    infolog.info('info')
    infolog.warning('警告')
    infolog.error('报错')
    infolog.critical('严重')
    
'''




import oss2
from .fileutil import list_files_deep
import os
class OSS_Class():
    def __init__(self, oss_path, ak, sk, ossBucket='projecteng', ossurl='http://oss-cn-shanghai.aliyuncs.com'):
        self.ak = ak
        self.sk = sk
        self.bucketName = ossBucket
        self.ossurl = ossurl
        self.oss_path = oss_path.strip()
        self.bucket = self.get_bucket()
    def get_bucket(self):
        print(f"当前空间地址:{self.oss_path}")
        auth = oss2.Auth(self.ak, self.sk)
        bucket = oss2.Bucket(auth, self.ossurl, self.bucketName)
        assert self.oss_path[0] not in ('/', "\\"), "路径第一位不需要'/';"
        assert self.oss_path[-1] == '/', "路径最后一位必须携带'/' "
        return bucket

    def get_current_folders(self):
        folders = []
        for obj in oss2.ObjectIterator(self.bucket,prefix=self.oss_path, delimiter = '/'):
            # print(obj)
            if obj.is_prefix():  # 判断obj为文件夹。
                folders.append(obj.key)
        return folders

    def get_bucket_filenames(self, suffix=''):
        all_names = [obj.key for obj in oss2.ObjectIterator(self.bucket, prefix=self.oss_path) if
                     obj.key.endswith(suffix)]
        if self.oss_path in all_names:
            all_names.remove(self.oss_path)
        # print(all_names)
        return all_names
    def downloadFile(self,filePath,saveFolder):
        # filePath 输入相对路径
        targetPath = self.oss_path + filePath.replace("\\", "/")
        self.bucket.get_object_to_file(targetPath,os.path.join(saveFolder, filePath))
        print(filePath+'download success')
    def deleteFile(self, files,headers=None):
        self.bucket.batch_delete_objects(files, headers)

    def upload_by_local_filename(self,local_file):
        base_name = os.path.basename(local_file)
        r= self.bucket.put_object_from_file(self.oss_path+base_name,local_file)
        if r.status != 200:
            raise Exception(f'oss返回状态不等于200 {r.status}')

    def upload_by_local_path(self, local_file_path, suffix='', log=None):
        # 检查前先提示，如果路径下的内容不为空，提示
        # 可接收消息队列，返回形式为列表[str,color]
        origin_filenames = self.get_bucket_filenames(suffix=suffix)
        if len(origin_filenames) != 0:
            print(f'Warning 当前目录不为空，已有 {len(origin_filenames)} 条数据，请确认！{origin_filenames}')
            if log: log(f'Warning 当前目录不为空，已有 {len(origin_filenames)} 条数据，请确认！{origin_filenames}', '#D4AF37')

        local_files = list_files_deep(local_file_path, suffix=suffix)
        target_files = [self.oss_path + os.path.relpath(x, start=local_file_path).replace("\\", "/") for x in
                        local_files]
        # print(local_files)
        # print(target_files)
        file_pares = zip(target_files, local_files)
        print(f"正在上传，请稍等 待上传总文件数:{len(local_files)}  上传后缀{suffix}")
        if log: log(f"正在上传，请稍等 待上传总文件数:{len(local_files)}  上传后缀{suffix}", "green")
        fileLength = len(local_files)
        for file_p,count in zip(file_pares,range(1,len(target_files)+1)):
            try:
                r = self.bucket.put_object_from_file(*file_p)
                if r.status != 200:
                    raise Exception(f'状态不等于200 {r.status}')

            except Exception as e:
                # print(file_p[0])
                print(e)
            print("上传进度:"+str(count)+'/'+str(fileLength))
            if log: log("上传进度:"+str(count)+'/'+str(fileLength), "green", "progressBar")
        print('上传完成，检查中，请稍等..')
        if log: log('上传完成，检查中，请稍等..', "green")

        uploaded = self.get_bucket_filenames(suffix=suffix)
        # checked = set(uploaded) ^ set(target_files)

        if len(origin_filenames) + len(local_files) != len(uploaded):
            print("\033[0;32;40m请注意！存在被覆盖图片\033[0m")
            if log: log("\033[0;32;40m请注意！存在被覆盖图片\033[0m", "#D4AF37")
        else:
            print(f"检查完毕，上传完成，上传总数量 {len(local_files)}")
            if log: log(f"检查完毕，上传完成，上传总数量 {len(local_files)}", "green")

    def set_obj_meta_and_check(self, suffix, meta_header, log=None):
        # 可接收消息队列，返回形式为列表[str,color]
        print(f"即将更新meta信息,后缀{suffix} {meta_header}")
        if log: log(f"即将更新meta信息,后缀{suffix} {meta_header}", "green")
        files = self.get_bucket_filenames(suffix)
        if len(files) != 0:
            print(f"发现 {suffix}文件数量： {len(files)},准备更新，执行过程可能较长，请耐心等待")
            if log: log(f"发现 {suffix}文件数量： {len(files)},准备更新，执行过程可能较长，请耐心等待", "green")
            fileLength = len(files)
            for count,filename in enumerate(files):
                self.bucket.update_object_meta(filename, headers=meta_header)
                print("更新进度:" + str(count) + '/' + str(fileLength))
                if log: log("更新进度:" + str(count+1) + '/' + str(fileLength), "green", "progressBar")
            print('更新完成 ,请进一步检查是否设置完整...')
            if log: log('更新完成 ,正在进一步检查是否设置完整...', "green")

            ### 执行检查过程
            error_files = []
            for count, filename in enumerate(files):
                meta = self.bucket.head_object(filename)
                metaheader = {k: v for k, v in meta.headers.items() if k in meta_header.keys()}
                if metaheader != meta_header:
                    error_files.append(filename)
                if log: log("检查进度:" + str(count+1) + '/' + str(fileLength), "green", "progressBar")
            if len(error_files) == 0:
                print("数据检查完毕，未发现问题")
                if log: log("数据检查完毕，未发现问题", "green")
            else:
                print(f"发现异常数据：{error_files}")
                if log: log(f"发现异常数据：{error_files}", "red")
        else:
            print(f"未发现{suffix}后缀文件")
            if log: log(f"未发现{suffix}后缀文件", "green")


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-14 09:16
Short Description:

Change History:

预处理必须的数据结构
{
  "instances": [
    {
      "id": "xxx", // 唯一编号勿重复
      "category": "CAR", // 请对应模板配置
      "number": 1, // 同一category下勿重复
      "children": [
        {
          "id": "yyy", // 唯一编号勿重复
          "name": "车身", // 请对应模板配置
          "number": 1, // 同一instance，同一name下勿重复
          "cameras": [
            {
              "camera": "default", // 请对应模板数据（若为单相机，则默认为default）
              "frames": [   //如果只有一个摄像头的话，这边。
                {
                  "frameIndex": 0,
                  "isKeyFrame": true, // 预标数据默认为关键帧true
                  "shapeType": "polygon",
                  "shape": {} // 图形数据（不同图形的数据结构不同，详情参考下方图形数据结构说明）
                }
              ]
            }
          ]
        }
      ]
    }
  ]
}



'''
import json
from peutils.transform.v1.base import *
from collections import defaultdict
from operator import itemgetter
from itertools import groupby
from peutils.transform.v1.base import ImgInstance,Img2Dobj
from peutils.textutil import gen_uuid
from peutils.datautil import GenCategorySeq

class ImgComPre():
    ### 提供ImgComFrame实例对象frame_list，或者提供frame_length构造一个空的
    def __init__(self,frame_length,instance_lst=None,raw_frames=None):

        self.frame_length = frame_length

        if raw_frames is not None:
            self.frames = [
                {
                    "frameIndex": f["frameIndex"],
                    "attributes": f.get("attributes",dict())
                }
                for f in raw_frames[0]["frames"]
            ]
        else:
            ### 构造一个空的属性
            self.frames = [
                {
                    "frameIndex": i,
                    "attributes": {}
                } for i in range(self.frame_length)]

        if instance_lst is not None:
            self.instance_lst = instance_lst
        else:
            self.instance_lst = []

        self.instance_seq = GenCategorySeq()
        self.imgobj_seq = GenCategorySeq()
        self.frameorder_seq = GenCategorySeq()

    def add_instance_obj(self,p_category,p_attributes=None):
        "p_id 生成p number序列 从1开始。 按照分类生成"
        ist_number = self.instance_seq.up_seq(p_category)
        ist = ImgInstance(
            id = gen_uuid(),
            number = ist_number,
            category = p_category,
            ist_attr = p_attributes if p_attributes else dict(),
        )
        self.instance_lst.append(ist)
        return ist

    def add_img_obj(self,instance,uuid,frameNum,c_category,shapeType,shape,c_attributes=None):
        # child_seq = self.instance_seq.up_seq(c_category)
        ### 先判断uuid + c_category 当前有没有

        ### 按每一帧进行编号
        frame_order = self.frameorder_seq.up_seq(str(frameNum))
        # print("frame_order",frame_order)

        obj = None
        for o in instance.obj_list:
            if o.id == uuid:
                obj = o
        if obj is None:
            # 识别到新的ID边用c_category 获取一个新的编号
            child_number = self.imgobj_seq.up_seq(c_category)
        else:
            child_number = self.imgobj_seq.get_seq(c_category)

        instance.obj_list.append(
                Img2Dobj(
                    instance=instance,
                    frameNum=frameNum,
                    id = uuid,
                    number = child_number,
                    category=c_category,
                    shapeType = shapeType,
                    shape = shape,
                    img_attr= c_attributes if c_attributes else dict(),
                    order = frame_order
                )
        )


    def dumps_data(self):
        _to_instances_dict = {
            "instances":[ i.to_pre_dict() for i in self.instance_lst],
            "frames":[
                {
                    "camera":"default",
                    "frames":self.frames
                }
            ]
        }
        instances_data = json.dumps(_to_instances_dict,ensure_ascii=False)
        return instances_data




if __name__ =="__main__":
    from pprint import pprint

    ### 之前数据的加载
    from peutils.transform.v1.img_com.parser import ImgComParse,ImgComDataConfig
    img = ImgComParse(
        # url = "https://oss-prd.appen.com.cn:9001/tool-prod/preview--r9nHFGv7vcbBW7P-JQ7t/preview--r9nHFGv7vcbBW7P-JQ7t.video-track-v2_task.video-track-v2_record.result.json",
        url="https://oss-prd.appen.com.cn:9001/tool-prod/a2a3ef0c-55c4-4d15-8cc2-ff6aeb7878dd/R.1650783983510.a2a3ef0c-55c4-4d15-8cc2-ff6aeb7878dd.CODU4AEQEg3d3d_2022-04-24T070156Z.18107.result.json",
        config=ImgComDataConfig(
        ))
    imgpre = ImgComPre(frame_length =img.frame_length ,
                       instance_lst=img.instance_lst,raw_frames=img.raw_data["frames"])
    print(imgpre.dumps_data())
    #### 加载之前的数据

    ### 创建新的数据
    from peutils.transform.v1.base import ImgInstance,Img2Dobj
    imgpre2 = ImgComPre(frame_length=img.frame_length)
    ist1 = imgpre2.add_instance_obj(p_category="小汽车",p_attributes={"a":1})

    ### id如果和之前相同，那么这里写入的时候会当成同一个物体写入，有tracking的功能。
    ### 这边的c_category 注意是平台这个子组建下的名称，不要放错到其他实例中
    ### 如果相同id 但是c_category 不一样那么重新创建一个child.
    imgpre2.add_img_obj(instance=ist1,uuid ="abc",frameNum=1,c_category="汽车头",shapeType="line",shape={"kk":1},c_attributes={"w":None})
    imgpre2.add_img_obj(instance=ist1, uuid="abc", frameNum=2, c_category="汽车头", shapeType="line2", shape={"kk": 2},
                        c_attributes={"w": "2"})

    ### 添加一个新的分类物体，但是标签相同
    imgpre2.add_img_obj(instance=ist1, uuid="dd", frameNum=1, c_category="汽车头", shapeType="line2", shape={"kk": 2},
                        c_attributes={"w": "2"})
    print(imgpre2.dumps_data())
    ### 先创建instance,写入这个instatnce所有的内容，再写入image对象。



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-02 22:12
Short Description:

Change History:

'''


from abc import ABC,abstractmethod
# import uuid
import os,sys
import json
import traceback
from peutils.transform.v1.base import ErrorMsgLogV1,gen_format_progress_seq,deco_execution_time
import pandas as pd
from concurrent import futures
from peutils.textutil import gen_uuid



## 单条数据的处理基类，可能是连续帧
class AbstractTask(ABC):

    def __init__(self,row,users_param=None):
        self.row= row # 原始csv这一行的数据
        self.users_param = users_param if users_param is not None else {} # 前端用户的参数
        self.log = ErrorMsgLogV1()  # 这行的所有错误
        # self.parse_obj = parse_cls(row["annotation"],config=config)

    @property
    def row_flag(self):
        if len(self.log.error_list)>0:
            return 0
        else:
            return 1

    @abstractmethod
    def check_row_on_a9(self)->None:
        """a9只能通过annotation进行检查 将内容输出到log"""

    @abstractmethod
    def check_row_addtional(self)->None:
        """额外的内容检查 将内容输出到log """

    def check_row_func(self,check_mode="all"): #check分a9的
        if self.log is None or isinstance(self.log,ErrorMsgLogV1) ==False:
            raise Exception("请定义log 并且使用ErrorMsgLog的实例")
        else:
            ### 将自带错误的解析方法放过去这个错误的方法
            # self.log.error_list.extend(self.parse_obj.check_frames_error())

            if check_mode=="a9_check":
                self.check_row_on_a9()
                return self.log.error_list  ##直接返回对象
            elif check_mode == "all":
                self.check_row_on_a9()
                self.check_row_addtional()
                return self.log.error_list ##做格式化
            else:
                raise Exception("未定义的check mode")


    @abstractmethod
    def parse_row_func(self)->None:
        """实现自定义的 parse_row_func方法  相当于单条数据的main函数"""



# def get_temp_path(by="nas"): # nas,oss
#
#     # 返回 需要授权的路径
#     pass

### 写一个方法，这个方法负责处理单行数据的逻辑
### csv所有的数据生成队列，按照并发去处理和完成，将所有的错误结果搜集，最后打印错误信息，并且输出到文件。
### 进度条功能
### OSS数据保存的功能

class CommonTaskV1():
    def __init__(self,row_cls:type,max_worker=16):
        self.RowTask = row_cls
        # self.parse_cls = parse_cls
        # self.config = config
        self.max_worker = max_worker
        self.format_progress = lambda: None


    def process_unit(self,row,users_param):
        row_task = self.RowTask(row=row,users_param=users_param)
        ## 执行检查逻辑,将错误输出到log

        ###执行方法,捕捉未知错误
        try:
            row_task.check_row_func()
            row_task.parse_row_func()
        except Exception as e:
            row_task.log.create_error(msg=repr(e))

        ### 这行所有错误的集合
        errors = row_task.log.fomart_error_str()

        return row_task.row_flag, row, errors

    ### 单个文件的处理
    @deco_execution_time
    def process_file(self,file,user_kw):
        ### 创建实例
        # row_task = self.RowTask(file=file,row_dict=)

        task_df = pd.read_csv(file,encoding='utf-8-sig')
        self.format_progress = gen_format_progress_seq(total=len(task_df), split_part=10)
        header = task_df.columns

        df_error = pd.DataFrame(columns=["errors",*header])
        # task_df = pd.DataFrame(columns=["col1", "col2"],
        #                        data=[{"col1": 1, "col2": "a"}, {"col1": 2, "col2": "b"},
        #                              {"col1": 3, "col2": "c"}, {"col1": 4, "col2": "d"}])

        with futures.ThreadPoolExecutor(min(self.max_worker, len(task_df))) as executor:

            tasks = [executor.submit(self.process_unit, row=row, users_param=user_kw) for _, row in task_df.iterrows()]

            for future in futures.as_completed(tasks):
                row_flag, row, errors, = future.result()  # flag 1代表是成功，0代表失败，失败后所有的数据，写入csv
                self.format_progress()
                if row_flag == 0:
                    df_error = df_error.append({"errors":errors,**row,},ignore_index=True)

            if len(df_error)>0:
                print(f"{file} 发现异常{len(df_error)}，请检查输出文件中的异常明细", file=sys.stderr)
                df_error.to_csv(f"error_{gen_uuid()}_{'.'.join(os.path.basename(file).split('.')[:-1])}.csv",index=False,encoding="utf-8-sig")
                # df_error.to_csv()


    ## process 方法的入口
    @deco_execution_time
    def process_func(self,files,**user_kw):

        if isinstance(files,list) == True:
            for file in files:
                self.process_file(file,user_kw)
        else:
            self.process_file(files,user_kw)








# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-13 11:20
Short Description:

Change History:

'''
from peutils.transform.v1.base import *
import json


class AudioSegFrame():
    def __init__(self,frameId,frameUrl,duration,
                 frame_attr,  #  对应audios中attributes 就是全局属性配置
                 config):
        self.frameId = frameId
        self.frameUrl = frameUrl
        self.duration = duration

        self.log = ErrorMsgLogV1()
        self.config = config
        self.frame_attr = frame_attr
        self.frame_obj_list = []


    def add_frame_obj(self,obj:AudioCutObj):
        ## 如果是连续模式，每次应该是等于上一个
        ## 如果是另外一种，每次的start应该是小于等于上一个
        if self.config.segment_mode =="continuous":
            if len(self.frame_obj_list) != 0:
                if obj.start != self.frame_obj_list[-1].end:
                    self.log.create_error("连续模式下新增切分段开始时间必须等于上一个结束时间",obj=obj)
        elif  self.config.segment_mode =="individual":
            if len(self.frame_obj_list) != 0:
                if obj.start < self.frame_obj_list[-1].start:
                    self.log.create_error("分段模式下，每个开始时间必须大于上一个结束时间",obj=obj)
        self.frame_obj_list.append(obj)


    def __repr__(self):
        return f'Frame {self.frameId} {len(self.frame_obj_list)}T'


class AudioSegParse(CommonBaseMixIn):

    ### 继承session属性 用来读取url
    def __init__(self,url,config):
        self.url = url
        self.config = config
        self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
        self.frames_lst, self.frame_length = self.parse_by_frame()

    def check_frames_error(self):
        all_errors = []
        for frame in self.frames_lst:
            all_errors.extend(frame.log.error_list)
        return all_errors

    def parse_by_frame(self):
        frames_lst = []
        for idx,raw_frame in enumerate(self.raw_data["audios"]):
            frame = AudioSegFrame(
                frameId= idx,
                frameUrl= raw_frame["url"],
                duration = raw_frame["duration"],
                frame_attr= raw_frame.get("attributes") if raw_frame.get("attributes") else dict(),
                config= self.config
            )
            frames_lst.append(frame)

        for idx,raw_frame_obj_list in enumerate(self.raw_data["results"]):
            for obj_idx, obj in enumerate(raw_frame_obj_list):
                audio_cut = AudioCutObj(
                    frameNum=idx,
                    id = obj["id"] ,
                    number = obj_idx + 1,
                    start = self.config.number_adpter_func(obj["start"]) if self.config.number_adpter_func else obj["start"],
                    end = self.config.number_adpter_func(obj["end"])if self.config.number_adpter_func else obj["end"],
                    block_attr= obj.get("attributes") if obj.get("attributes") else dict(),
                    line_contents= obj["content"],
                )
                frames_lst[idx].add_frame_obj(audio_cut)

        return frames_lst,len(frames_lst)


class AudioSegDataConfig():

    def __init__(self,segment_mode,parse_id_col="id",number_adpter_func=None,seq_start=0):
        assert segment_mode in {"continuous","individual"},"分段模式必须是 continuous 或者 individual "
        self.segment_mode = segment_mode  # continuous,individual
        self.number_adpter_func = number_adpter_func
        self.parse_id_col = parse_id_col, # 默认id ## 暂时不引入fid,gid
        self.seq_start = seq_start  # 暂时用不到


### 界面会根据顺序排序的

if __name__ =="__main__":
    ## 单帧
    from pprint import pprint
    ad = AudioSegParse(url="https://oss-prd.appen.com.cn:9001/tool-prod/preview-IvwN9qwWAujQfAbtkqWMw/preview-IvwN9qwWAujQfAbtkqWMw.long-audio_task.long-audio_record.result.json",
                         config =AudioSegDataConfig(
                             segment_mode="continuous",
                             number_adpter_func = lambda i: round(i,5)
                         ))
    # pprint(ad.frames_lst[0].frame_obj_list)

    for cut in ad.frames_lst[0].frame_obj_list:
        print(cut.start,cut.end,cut.line_contents[0]["text"])


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-05 15:53
Short Description:

Change History:

'''

def unique(iterable, key=None):
    seen = set()
    for item in iterable:
        val = item if key is None else key(item) #要看后面key的函数传的什么，也是转成元组比较
        if val not in seen:
            yield item
            seen.add(val)

## list unique
'''
a = [1, 5, 2, 1, 9, 1, 5, 10]
print(list(unique(a)))
'''

'''
a = [{'x':1, 'y':2}, {'x':1, 'y':3}, {'x':1, 'y':2}, {'x':2, 'y':4}]
list(unique(a, key=lambda d: (d['x'],d['y'])))
'''

## file unique
'''
with open("/Users/hwang2/Documents/a.txt") as f:
    for i in unique(f):
        print(repr(i))
'''

## csv unique
'''
import csv
from operator import itemgetter
with open("/Users/hwang2/Documents/b.csv") as f:
    for row in unique(csv.reader(f),key=itemgetter(0,1)):
        print(row)
'''



def gen_uuid_seq(start=0):
    '''
    start 等于-1 第一个值出现值是0
    start 等于 0 第一个出现值是1
    '''
    # 0的话就代表 从1 开始
    count = start
    uuid_dict=dict()
    def new_uuid(uuid):
        nonlocal count
        nonlocal uuid_dict
        if uuid not in uuid_dict:
            count += 1
            uuid_dict[uuid]=count
        # print(uuid_dict,count)
        return uuid_dict[uuid]
    return new_uuid

'''
def my_uuid():
    from random import choice
    for i in range(10):
        rd = choice(['a','b','c','d'])
        print(rd,get_uuid_seq(str(rd)))

for i in range(3):
    print(f'round {i}')
    get_uuid_seq = gen_uuid_seq()
    my_uuid()

'''

class GenCategorySeq():
    def __init__(self,start=0,default_dict=None):
        self.start = start
        self.uuid_dict = dict() if default_dict is None else default_dict

    def up_seq(self,uuid):
        if uuid not in self.uuid_dict:
            self.uuid_dict[uuid] = self.start +1
        else:
            self.uuid_dict[uuid] = self.uuid_dict[uuid]+1

        return self.uuid_dict[uuid]

    def get_seq(self,uuid):
        return self.uuid_dict.get(uuid,self.start)
'''
不带初始数据
category_seq = GenCategorySeq(start=0)
print(category_seq.up_seq("a"))
print(category_seq.up_seq("b"))
print(category_seq.up_seq("a"))
1
1
2

'''

'''
带初始数据
category_seq = GenCategorySeq(default_dict={"a":10})
print(category_seq.up_seq("a"))
print(category_seq.up_seq("b"))
print(category_seq.up_seq("a"))

11
1
12
'''




# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-09-22 11:19
Short Description:

Change History:

'''
# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-09-02 14:09
Short Description:

Change History:




'''

import asyncio
import aiohttp
import uvloop
import os,sys
import pandas as pd
import json
import csv


asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())


def gen_uuid():
    import uuid
    uid = str(uuid.uuid4())
    suid = ''.join(uid.split('-'))
    return suid

def saveCsvData(savePath, data: list, header=None, newline='\n'):
    csv.field_size_limit(500 * 1024 * 1024)
    with open(savePath, 'w', encoding='utf-8-sig', newline=newline) as f:
        writer = csv.writer(f, quoting=csv.QUOTE_ALL, lineterminator="\n")
        if header:
            writer.writerow(header)
        writer.writerows(data)


'''
输入 
    a9的csv 
    a9不支持其他参数的输入
'''




class A9CommonDataParseV1():
    project_desc = None
    need_process_suffix = ".csv"
    # other_params =dict()

    cols = ["Batch Num", "Record ID", "annotation"]
    parse_url = None

    ###以下参数一般不要改
    headers = {"Content-Type": "application/json"}
    async_task_nums = 1
    mode="parse"



    def __init__(self):
        self.temp_dir,self.work_dir = self.get_temp_dir_and_work_dir()
        self.resource_file = sys.argv[1]


    def get_temp_dir_and_work_dir(self):
        temp_dir = os.getcwd()
        resut_dir = os.path.join(temp_dir, "result")
        return temp_dir, resut_dir

    # 获取所有需要处理的任务，然后把任务加到任务池
    def get_all_tasks(self,taks_files):
        for file in taks_files:
            df = pd.read_csv(file,encoding='utf-8-sig')
            for _,row in df.iterrows():
                yield file, row,None # 最后一个是split_frame_idx

    async def request_data(self,session, url, data):
        async with session.post(url, json=data, headers=self.headers) as resp:
            # print(resp.status)
            rs = await resp.text()
            return resp.status, rs

    async def request_data_with_retry(self,session, url,data):
        tries = 3
        while tries > 0:
            try:
                return await self.request_data(session, url, data)
            except aiohttp.ClientResponseError:
                if tries <= 0:
                    raise Exception("请求错误超过3次")
            tries -= 1

    ### 执行单个异步任务，根据具体的业务覆盖这个方法
    async def process_func(self,file,task_row,split_frame_idx,semaphore):
        async with semaphore:
            async with aiohttp.ClientSession() as session:
                payload = {
                    "file":file,
                    "data":{
                        "row_dict": task_row[self.cols].to_dict(),
                        "split_frame_idx": split_frame_idx,
                    },
                    "mode":self.mode,
                }
                statu,rs = await self.request_data_with_retry(session, url=self.parse_url, data=payload)

                return file,task_row,statu,rs  # 文件名，原始行数据，以及API返回的结果

    def get_process_task_files(self,inputfilename):

        assert self.need_process_suffix.endswith(".csv")==True,"必须是csv格式"
        if inputfilename.endswith(self.need_process_suffix):
            return [inputfilename]
        else:
            return []


    async def process_all_tasks(self,task_files):
        semaphore = asyncio.Semaphore(self.async_task_nums)
        task_list = []

        print("获取任务明细...")
        for file,task_row,split_frame_idx in self.get_all_tasks(task_files):
            task = asyncio.create_task(self.process_func(file,task_row,split_frame_idx,semaphore))
            task_list.append(task)
        print("总任务数量:",len(task_list))

        res = await asyncio.gather(*task_list)
        return res

    # @deco_execution_time
    def check_all_tasks_result(self):
        pass

    # @deco_execution_time
    def handler_tasks_result(self):
        pass

    def save_rs_on_a9(self,reloutdir,outfilename,json_str):
        json_dict = json.loads(json_str)
        os.makedirs(reloutdir,exist_ok=True)
        with open(os.path.join(reloutdir,outfilename),'w',encoding='utf-8') as fw:
            json.dump(json_dict,fw,ensure_ascii=False,indent=4)




    def main(self):
        print("start ...")
        task_files = self.get_process_task_files(self.resource_file)
        # task_files = self.get_process_task_files("/Users/hwang2/Downloads/result_report_A4141-Li66571_2021-08-03T08_00_07.010205Z[UTC].csv")

        # 异步执行所有任务并拿到结果
        print("ready to process taskfiles",task_files)
        res = asyncio.run(self.process_all_tasks(task_files))


        # 执行结果分析和处理 r如果有error 把错误写到csv文件中
        error_header = ["file","record_id","batch_num","image_url",'annotation','http_code',"errors"]
        error_lines = []

        for file,task_row,statu,rs in res:
            # print(file,task_row,statu,rs)
            record_id = task_row["Record ID"]
            batch_num = task_row["Batch Num"]
            image_url = task_row.get("image_url")
            annotation = task_row.get("annotation")
            http_code = statu
            if statu !=200:
                errors = "网络或接口异常"
                error_lines.append([file,record_id,batch_num,image_url,annotation,http_code,errors])

            else:
                rs_dict = json.loads(rs)
                if rs_dict["code"]!=200:
                    errors = json.dumps(rs_dict["errors"],ensure_ascii=False)
                    error_lines.append([file,record_id, batch_num, image_url, annotation, http_code, errors])
                else:
                    self.save_rs_on_a9(rs_dict["output"]["outdir"],rs_dict["output"]["outfilename"],rs_dict["output"]["json_str"])


        if len(error_lines)==0:
            print("未发现异常")
        else:
            print(f"发现异常{len(error_lines)}，请检查输出文件中的异常明细",file=sys.stderr)
            saveCsvData(os.path.join(self.temp_dir,f"ERROR_{gen_uuid()}.csv"),error_lines,error_header)




# class A9CommonAfter(A9CommonDataParseV1):
#     project_desc = "毫末2D鱼眼车道线后处理"
#     # need_process_suffix = ".csv"  # A9不支持其他 不需要改
#
#     # other_params = dict() #默认参数 不支持其他参数，不要改
#
#
#     cols = ["annotation","json_name","raw_json_result"]
#
#     # api_use_cache = False # 默认是False  默认即可不要改
#
#
#     # save_way = "" # A9没有保存差异的方式
#
#
#     # async_task_nums = 10  # 默认10 一般不需要改
#     parse_url = f"https://process-api.appen.com.cn/haomo-2d-fisheye"



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-08 15:44
Short Description:

Change History:

'''
from peutils.transform.v1.base import *
import json
from peutils.datautil import gen_uuid_seq


class LidarPointFrame():
    def __init__(self,frameId,frameUrl,isValid,isSpecial,frameUrlInternal,frameUrlExternal,totalPointCount,
                 imageAttributes,
                 frame_attr,items,images,
                 config):
        self.frameId = frameId
        self.frameUrl = frameUrl
        self.isValid = isValid
        self.isSpecial = isSpecial
        self.frameUrlInternal = frameUrlInternal
        self.frameUrlExternal = frameUrlExternal
        self.totalPointCount = totalPointCount
        self.log = ErrorMsgLogV1()

        self.config = config
        if self.config.parse_id_col =="fid":
            self.config.seq_func = gen_uuid_seq(start=self.config.seq_start)

        self.frame_attr = frame_attr
        self.imageAttributes = imageAttributes
        self.lidar_dict,self.real_count = self.get_lidar_dict(items)

        if self.config.leak_check ==True:
            if self.totalPointCount != self.real_count:
                self.log.create_error(f"帧{self.frameId},点云未全部标注")

        self.camera_list,self.camera_meta,self.images_dict,self.images_list= self.get_images_dict(images)

        ### 如果是全景标注，可以生成一个完整的category对象

        self._img_idset = self.get_img_idset(self.images_dict)
        self.only_lidar_idset= self.lidar_dict.keys() - self._img_idset # 只有3D 没有出现在2D的ID
        self.only_image_idset = self._img_idset -  self.lidar_dict.keys() # 只出现在2D没有出现在3D中的ID
        self.has_23d_idset = self.lidar_dict.keys() & self._img_idset # 2D和3D都出现



    def get_fullpoint_map_list(self):
        if self.config.leak_check ==False and self.config.fill_category is None:
            raise Exception("未开启漏点检查的情况下，必须提供补充的category,否则不可使用此方法")

        # [{"pidx":0,"category":"地面"}] # 按照原始点的顺序进行排序.
        ### 如果有fill_category,允许补标签
        out_list = []
        seen_idx = set()
        for k,v in self.lidar_dict.items():
            category = v.category
            for p in v.points:
                out_list.append({
                    "pidx":p,
                    "category":category
                })
                seen_idx.add(p)
        ### 补其他点
        leak_idx = {x for x in range(self.totalPointCount)} - seen_idx
        for lk in leak_idx:
            out_list.append({
                "pidx":lk,
                "category":self.config.fill_category
            })
        #检查点的数量
        if len(out_list) != self.totalPointCount:
            self.log.create_error(f"{self.frameId}帧 补点数量不正确")

        out_list.sort(key=lambda x : x["pidx"]) ## 排序非常关键
        return out_list



    @staticmethod
    def get_img_idset(images_dict):
        img_idset = set()
        for _,v in images_dict.items():
            img_idset = img_idset | v.keys()
        return img_idset

    def get_img_obj_list(self,id,keep_nan=False):
        lst = []
        for cam in self.camera_list:
            obj = self.images_dict[cam].get(id)
            if keep_nan == True:
                lst.append(obj)
            else:
                if obj:
                    lst.append(obj)
        return lst


    def parse_lidar_by_item(self,item):

        '''
        检查逻辑
        1.如果yaw_only True,检查 rotation x 和 y都是0
        2.检查points长度和pointCount一样
        '''
        lidar_obj = LidarPointObj(
            frameNum = self.frameId,
            id = item["id"],
            category=item["category"],
            number=item["number"],
            points= item["points"],
            point_attr = json.loads(item["labels"]) if item.get("labels") else dict(),
            pointsLabels=item["pointsLabels"],
            pointCount = item["pointCount"],
            type = item["type"],
        )

        if self.config.parse_id_col =="id":
            key = item["id"]
        elif self.config.parse_id_col in ("gid","fid"):
            key = self.config.seq_func(item["id"])
        else:
            raise Exception("parse_id_col解析模式错误")

        ####业务检查
        if len(item["points"]) != item["pointCount"]:
            self.log.create_error(msg="点数量和数组不一致",obj=lidar_obj)


        return key,lidar_obj,item["pointCount"]

    def parse_img_by_item(self,item,width,height):

        '''
        检查逻辑
        1.图像是否超出边界外

        '''
        if item["type"] =="polyline":
            img_obj = LidarPointPolyline(
                frameNum=item["frameNum"],
                imageNum=item["imageNum"],
                id=item["id"],
                number=item["number"],
                type=item["type"],
                category=item["category"],
                points=item["points"],
                img_attr=json.loads(item["labels"]) if item.get("labels") else dict(),
            )
        else:
            # 暂时不会遇到
            img_obj =None

        if self.config.parse_id_col == "id":
            key = item["id"]
        elif self.config.parse_id_col in ("gid","fid"):
            key = self.config.seq_func(item["id"])
        else:
            raise Exception("parse_id_col解析模式错误")

        return key, img_obj


    def get_lidar_dict(self,items):
        lidar_dict = dict()
        real_count = 0

        for item in items:
            key,lidar_obj,pointCount =self.parse_lidar_by_item(item=item)
            lidar_dict[key] = lidar_obj
            real_count += pointCount

        if len(items) != len(lidar_dict):
            raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
        return lidar_dict,real_count

    def get_single_image_dict(self,items,width,height):
        single_image_dict = dict()

        for item in items:
            if item["type"] =="polyline":
                key,img_obj = self.parse_img_by_item(item,width,height)
                single_image_dict[key] = img_obj
            else:
                raise Exception("目前图像物体仅支持polyline,其他类型还在开发中")

        if len(items) != len(single_image_dict):
            raise Exception(f"{self.config.parse_id_col} 解析模式下数量不等，请检查使用参数")
        return single_image_dict

    def get_images_dict(self,images):
        ### 先解析出镜头数量
        images_dict = dict() #{cam:dict1,cam2:dict2,...}
        images_list = [] #[dict1,dict2,..]
        camera_list = []
        camera_meta = dict()

        for img in images:
            image_path = img["image"]
            camera_name = image_path.split("/")[-2]
            camera_list.append(camera_name)

            camera_meta[camera_name]= {
                "image_path":image_path,
                "imageUrlInternal":img["imageUrlInternal"],
                "imageUrlExternal":img["imageUrlExternal"],
                "width": img["width"],
                "height": img["height"],

            }
            sg_img_dict = self.get_single_image_dict(img["items"],width=img["width"],height = img["height"])
            images_dict[camera_name] = sg_img_dict
            images_list.append(sg_img_dict)

        ## check
        if len(camera_list) != len(images):
            raise Exception("找到的镜头数量和原始不一致")

        return camera_list,camera_meta,images_dict,images_list


    def __repr__(self):
        return f'Frame {self.frameId}: {self.totalPointCount}T {self.real_count}P'



class LidarPointParse(CommonBaseMixIn):

    ### 继承session属性 用来读取url
    def __init__(self,url,config):
        self.url = url
        self.config = config
        if self.config.parse_id_col == "gid":
            self.config.seq_func = gen_uuid_seq(start=self.config.seq_start)

        self.raw_data = self.get_raw_data(url) # 获取JSON字典数据数据
        self.frames_lst,self.frame_length = self.parse_by_frame()


    def check_frames_error(self):
        all_errors = []
        for frame in self.frames_lst:
            all_errors.extend(frame.log.error_list)
        return all_errors


    def parse_by_frame(self):
        '''
        返回解析后的frame列表，对应的数据
        '''
        #
        frames_lst = []
        for raw_frame in self.raw_data["results"]:

            ### 属性
            attribute = raw_frame.get("attribute")
            if attribute:
                frame_attr = json.loads(attribute)
            else:
                frame_attr = dict()

            frame = LidarPointFrame(
                frameId=raw_frame["frameId"],
                frameUrl=raw_frame["frameUrl"],
                isValid= raw_frame["attributes"]["valid"],
                isSpecial =raw_frame["attributes"]["special"],
                frameUrlInternal=raw_frame["frameUrlInternal"],
                frameUrlExternal=raw_frame["frameUrlExternal"],
                totalPointCount=raw_frame["totalPointCount"],
                frame_attr = frame_attr,
                items = raw_frame["items"],
                images = raw_frame["images"],
                imageAttributes=raw_frame["imageAttributes"],
                config= self.config
            )
            frames_lst.append(frame)
        return frames_lst,len(frames_lst)


class LidarPointDataConfig():
    def __init__(self,leak_check=False,number_adpter_func=None,fill_category=None,
                 parse_id_col="id",seq_start=0,overflow=False):
        self.leak_check = leak_check # 为True的时候会检查点是否完整
        self.number_adpter_func = number_adpter_func
        self.fill_category = fill_category
        self.parse_id_col = parse_id_col # id或g_id, frame_id  # 这里没有number选项，因为默认是分类编号
        self.seq_start = seq_start
        self.overflow = overflow # 这里暂时没做检查

from pprint import pprint

if __name__ =="__main__":
    ## 单帧
    from pprint import pprint
    lidarpoint = LidarPointParse(url="https://oss-prd.appen.com.cn:9001/tool-prod/2bcdc0ebda8758cf9caeb29ba7cfa408/R.1654659117117.lidar.seg.2df039af-a185-49cf-84e7-be8b295b7077.COSPsAIQDA3d3d_2022-06-08T033126Z.14513.QA_RW.f8d93205-4c8f-47ee-9a3c-3ae80f7d90e5.8308b8d5ec23225be41513692c55b8cc.review.json",
                         config =LidarPointDataConfig(
                             leak_check = False,  ## 检查点的数量是否和总点的数量一样.
                             fill_category= "背景", # 对于未标注的数据默认的标签，如果不需要的时候就是None
                             number_adpter_func=None, #lambda i: round(i,3), # 默认None
                             parse_id_col = "id",
                             seq_start = 0, # 如果是 gid,fid,或者frame_id 需要提供seq_start， 如果是0就是从1开始编号，如果是-1就是从0开始编号
                             overflow= False
                         ))
    pprint(lidarpoint.frames_lst[0])#.lidar_dict)
    pprint(lidarpoint.frames_lst[0].lidar_dict)
    print(lidarpoint.frames_lst[0].log.error_list)
    pprint(lidarpoint.frames_lst[0].images_dict)

    # pprint(lidarpoint.frames_lst[0].get_fullpoint_map_list()) # 获取点的category排序后的完整结果
    # print(lidarpoint.frames_lst[0].totalPointCount)



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2020-11-10 16:41
Short Description:

Change History:

'''

### 解压文件
def unzipfile(zip_file_path):
    import zipfile
    import os
    dir_name = os.path.dirname(zip_file_path)
    with zipfile.ZipFile(zip_file_path, 'r') as z:
        for x in z.filelist:
            if x.filename.split("/")[-1].startswith('.')==False:
                z.extract(x.filename, dir_name)
    return dir_name


### 指定输出路径和列表，打包路径
def packzipfile(outpath,pack_name_list):
    import zipfile
    import os
    with zipfile.ZipFile(outpath, 'w') as myzip:
        for fname in pack_name_list:
            myzip.write(fname,os.path.basename(fname))



from argparse import ArgumentParser,FileType
import zipfile
import sys
import os
from .fileutil import list_files_deep


'''
支持单个压缩包文件中存放单种类型的文件(不支持zip种存放zip)
支持文件逐一处理

def process_func(filename,**kw):
    print(filename,kw)
    print(kw["arg1"],kw["choicearg2"],kw["optionarg"])

other_params ={
    "arg1":{
        "help":"arg1",
        "type":str,
    },
    "choicearg2":{
        "help":"i am a choice",
        "choices":["a","b","c"],

    },
    "-optionarg":{
        "help":"option args",
        "type":str
    },
}

h = WooeyBaseZipFileiHander("test template with zip class",".csv",process_func=process_func,other_params=other_params)


'''


class WooeyBaseZipHandlerFile():
    '''
    project_desc: str
    need_process_suffix : str
    process_func function
    other_params a list like {"arganeme":{}} if name start with -- means option param
    {
        "arg1":{
            "help:"arg help", #not empty
            "type":int ,# not empty when it not choices.
            "default":'', #option
            "choices":['a','b','c'] #option

        },
        "--arg2":{}, option arg2
        ...
    }
    ##
    '''
    def __init__(self,project_desc,need_process_suffix,process_func,other_params=None):
        self.project_desc=project_desc
        self.need_process_suffix = need_process_suffix
        self.process_func = process_func
        self.other_params = other_params

    def main(self):
        parser = ArgumentParser(description=self.project_desc)
        parser.add_argument('inputfile', help='Upload your file can be sigle file or a zipfile include mutifiles',
                            type=FileType('r'))
        if self.other_params:
            for k,v in self.other_params.items():
                parser.add_argument(k,**v)

        args = parser.parse_args()
        filename = args.inputfile.name
        assert self.need_process_suffix.endswith(".zip")==False,"内容文件不能是zip格式"

        # add other param
        users_param = {}
        if self.other_params:
            for k,v in self.other_params.items():
                # 移除非必须参数前面的-
                users_param[k] = getattr(args,k.lstrip("-") )


        if filename.endswith(self.need_process_suffix):
            self.process_func(filename,**users_param)

        elif filename.endswith(".zip"):
            print("ready to extract file")
            dir_name = os.path.dirname(filename)
            with zipfile.ZipFile(filename, 'r') as z:
                z.extractall(path=dir_name)
            print("extract finish... ready to process")
            # 解压完成后，读取文件
            files = list_files_deep(dir_name, suffix=self.need_process_suffix)
            print(f"发现待处理文件{len(files)}")

            for file in files:
                print(f"process file {file}")
                self.process_func(file,**users_param)
        else:
            raise Exception(f"file name suffix not endswith {self.need_process_suffix} or not zipfile")

'''
支持单个压缩包文件中存放单种类型的文件(不支持zip种存放zip)
文件批量读取后一起处理

def process_func(filelists,**kw):
    pass
'''

class WooeyBaseZipHandlerFileList():
    def __init__(self, project_desc, need_process_suffix, process_func, other_params=None):
        self.project_desc = project_desc
        self.need_process_suffix = need_process_suffix
        self.process_func = process_func
        self.other_params = other_params

    def main(self):
        parser = ArgumentParser(description=self.project_desc)
        parser.add_argument('inputfile', help='Upload your file can be sigle file or a zipfile include mutifiles',
                            type=FileType('r'))
        if self.other_params:
            for k, v in self.other_params.items():
                parser.add_argument(k, **v)

        args = parser.parse_args()
        filename = args.inputfile.name
        assert self.need_process_suffix.endswith(".zip") == False, "内容文件不能是zip格式"

        # add other param
        users_param = {}
        if self.other_params:
            for k, v in self.other_params.items():
                # 移除非必须参数前面的-
                users_param[k] = getattr(args, k.lstrip("-"))

        if filename.endswith(self.need_process_suffix):
            self.process_func([filename], **users_param)  ### 这里转变成列表

        elif filename.endswith(".zip"):
            print("ready to extract file")
            dir_name = os.path.dirname(filename)
            with zipfile.ZipFile(filename, 'r') as z:
                z.extractall(path=dir_name)
            print("extract finish... ready to process")
            # 解压完成后，读取文件
            files = list_files_deep(dir_name, suffix=self.need_process_suffix)
            print(f"发现待处理文件{len(files)}")

            self.process_func(files,**users_param)  ## 这里传入多个文件

        else:
            raise Exception(f"file name suffix not endswith {self.need_process_suffix} or not zipfile")




# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-14 17:45
Short Description:

Change History:

'''
from peutils.pcd_py3 import PointCloud
import numpy as np


def get_pointlist_from_path(path,fltype=None,dtype=None):
    # path可以是当时标注的点云对应文件，也可以是客户标注的对应文件，如果是客户的文件，我们要注意是否有去过Nan点，这种需要补点.
    # path可以是url 也可以是路径

    if fltype not in ("pcd","bin"):
        raise Exception("fltype必须是pcd或者bin类型")
    if fltype =="bin":
        if dtype is None:
            raise Exception("bin文件必须指定数据类型")

    if fltype =="bin":
        bin_data = np.fromfile(path,dtype=dtype)
        return bin_data
    elif fltype =="pcd":
        dt = PointCloud.from_path(path)
        return dt.pc_data

def get_pointlist_from_url(session,url,fltype=None,dtype=None):

    if fltype not in ("pcd","bin"):
        raise Exception("fltype必须是pcd或者bin类型")
    if fltype =="bin":
        if dtype is None:
            raise Exception("bin文件必须指定数据类型")

    bin_r = session.get(url, stream=True)
    if fltype == "bin":
        bin_data = np.frombuffer(bin_r.content, dtype=dtype)
        bin_r.close()
        return bin_data  # np的一维数组
    elif fltype == "pcd":
        dt = PointCloud.from_buffer(bin_r.content)
        bin_r.close()
        return dt.pc_data  # 通过 x y z 等取数据

'''
ss = get_session()
p_list = get_pointlist_from_url(ss,"https://appen-storage.oss-cn-shanghai.aliyuncs.com/humor_3d/bucket-dataengine/release/preparation/card_data/icu30/data_label_preprocess/619c8ea93842dacdbf91b66e/center_128_lidar/1636006959490492.pcd",
                                    fltype="pcd")
for p in p_list:
    print(p["x"],p["y"],p["z"],p["intensity"])

p_list = get_pointlist_from_path("/Users/hwang2/Downloads/1636006959490492.pcd",fltype="pcd")
for p in p_list:
    print(p["x"],p["y"],p["z"],p["intensity"])

p_array = get_pointlist_from_url(ss,"https://projecteng.oss-cn-shanghai.aliyuncs.com/haomo_3d/segmentation/20210602/HDR006_60b0599b50eb3fa074e490d9_3034_QA/point_cloud/bin_v1/1620539160100000.bin",
            fltype="bin",dtype="float32")
p_array = get_pointlist_from_path("/Users/hwang2/Downloads/1620539160100000.bin", fltype="bin",dtype="float32")
print(p_array[:10])

'''



# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-05 13:20
Short Description:

Change History:

'''

from setuptools import setup, find_packages



setup(
    name = "peutils",
    version = '0.0.24',
    keywords = ("pip3", "peutils",'henry'),
    description = "utils for text,audio,video,excel,file and so on.",
    long_description = "utils for text,audio,video,excel,file and so on,more details please visit gitlab.",
    license = "MIT Licence",
    url = "https://github.com/yunsansheng/peutils",
    author = "henry.wang",
    author_email = "shanandone@qq.com",
    packages = find_packages(),
    include_package_data = True,
    platforms = "any",
    install_requires = []
)




# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-03-17 11:18
Short Description:

Change History:

'''
import numpy as np
import cv2
import re
import shutil
import base64
import os




class DrawMaskCls:
    def __init__(self, image_path,width,heigth, objects_data, result_path, colorMapping=None):
        '''

        :param image_path: local image path
        :param objects_data:
        :param result_path:
        :param colorMapping:
        输入颜色格式:
        {
            "行人":"#68bc00",
            "sse-eraser":"#ffffff"
            ...
        }
        '''
        self.image_path = image_path
        self.width = width
        self.height = heigth
        self.objects_data = objects_data
        self.result_path = result_path
        self.colorMapping = colorMapping


    def get_image_meta(self, image_path):
        img_data = cv2.imread(image_path)
        # img_data = cv2.imdecode(np.fromfile(image_path,dtype=np.uint8), -1)
        # 2021-2-7-21:38 xtu changed:
        # 增加win10旋转，目前尝试PIL以及PyPlot,numpy读取文件不会对其进行处理，需要旋转，cv2为已翻转信息，不能使用翻转！！
        # 旋转度信息为0112,仅针对exif信息，有可能图片不存在exif信息,png图片不存在该bug

        # https://blog.csdn.net/m1m2m3mmm/article/details/78401523
        h, w = img_data.shape[0], img_data.shape[1]
        return h, w


    def gen_mask_layer(self):
        # print(self.objects_data)
        # 生成空图
        mask_layer = np.zeros((self.height, self.width, 3), np.uint8)

        '根据mask顺序生成数据'

        for object in self.objects_data:
            # 获取坐标,二维数组。 [[x,y],[x,y]]
            points_np = np.array([self.getObjPoints(object)], dtype=np.int32)

            # 获取名称lassName
            objectClassName= object["className"]
            objectColorCode = object['classColor']  #"#009ce0"

            # 获取颜色rgb_list [0,0,0] # 范围是0-256
            if self.colorMapping is None:
                if objectClassName == 'sse-eraser':
                    rgb_list = [0, 0, 0]
                else:
                    rgb_list = self.transfer_color_code(objectColorCode)
            else:
                rgb_list = self.transfer_color_code(self.colorMapping[objectClassName])

            # 填充信息形状和颜色信息
            cv2.fillPoly(mask_layer, points_np, rgb_list)

        # 预览
        # cv2.imshow('URL2Image', black_backgroud)
        # cv2.waitKey()
        return mask_layer

    @staticmethod
    def transfer_color_code(color_str:str)->list:
        '''
        #ffffff 格式转成list [255,255,255]
        :param color_str:
        :return:
        '''
        return [int(color_str[5:7], 16), int(color_str[3:5], 16), int(color_str[1:3], 16)]


    @staticmethod
    def getObjPoints(obj):
        # 一纬数组
        return [[point['x'], point['y']] for point in obj['polygon']]


    def combine_mask_and_origin(self, maskOpacity=0.5, orginOpacity=0.8):
        mask_layer = self.gen_mask_layer()
        orgin_layer = cv2.imread(self.image_path)
        # orgin_layer = cv2.imdecode(np.fromfile(self.image_path,dtype=np.uint8),-1)
        # # imdecode读取的是rgb，如果后续需要opencv处理的话，需要转换成bgr，转换后图片颜色会变化
        # orgin_layer = cv2.cvtColor(orgin_layer, cv2.COLOR_RGB2BGR)
        combine_layer = cv2.addWeighted(orgin_layer, orginOpacity, mask_layer, maskOpacity, 1)
        return combine_layer


    def main_result(self, mask=True, combine=True, origin=True, jsondata=True):
        image_name = os.path.basename(self.image_path)
        image_name_without_suffix = '.'.join(os.path.basename(self.image_path).split(".")[:-1])

        if mask:
            mask_layer = self.gen_mask_layer()
            out_path = os.path.join(self.result_path,"mask")
            os.makedirs(out_path,exist_ok=True)
            cv2.imencode('.png', mask_layer)[1].tofile(os.path.join(out_path, image_name_without_suffix+'.png'))

        if combine:
            combine_layer = self.combine_mask_and_origin()  # 使用默认配置
            out_path = os.path.join(self.result_path, "mask_on_origin")
            os.makedirs(out_path,exist_ok=True)
            cv2.imencode('.png', combine_layer)[1].tofile(os.path.join(out_path, image_name_without_suffix + '.png'))


        if origin:
            out_path = os.path.join(self.result_path, "origin")
            os.makedirs(out_path,exist_ok=True)
            shutil.copyfile(self.image_path, os.path.join(out_path, image_name))







# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2021-09-18 13:02
Short Description:

Change History:

'''

from abc import ABC,abstractmethod
import uuid
import os
import json
import traceback

def gen_uuid():
    uid = str(uuid.uuid4())
    suid = ''.join(uid.split('-'))
    return suid

class ErrorMsgLog():

    def __init__(self):
        self.error_list = []

    def create_error(self,msg,obj_id=None,group_id=None,only_group=False,frame_lst=None,block=True):
        '''
        如果plss模版，那么可以增加"info":{objectId:xx,groupId:xx}
        其他的直接使用obj_id即可
        '''
        plss_info=dict()

        if obj_id is None:
            obj_id = "common_" + gen_uuid()
        else:
            plss_info["objectId"] = obj_id

        if group_id is not None:
            plss_info["groupId"] = group_id

        if frame_lst is None:
            frame_lst = [0]

        err = {
            "id":obj_id,
            "message":msg,
            "frames":frame_lst,
            "blockSubmit":block
        }

        if plss_info !=dict():
            err["info"] = plss_info

        self.error_list.append(err)

    @staticmethod
    def single_errro(e)->str:

        base = f'''Id:{e["id"]}, Msg:{e["message"]}'''
        #如果frame是0，不打印。
        if e["frames"]==[0]:
            pass
        else:
            base += f", Frame:{repr(e['frames'])}"
        if e.get("info") is not None:
            if e["info"].get("groupId") is not None:
                base += f", Group:{e['info']['groupId']}"

        return base


    def fomart_error(self)->[]:
        return [self.single_errro(e) for e in self.error_list]
        # 如果frame是0，不打印。


class AbstractProcess(ABC):
    log = None

    def __init__(self,file,row_dict, users_param=None,split_frame_idx=None):
        self.file = file
        self.row_dict = row_dict
        self.users_param = users_param if users_param is not None else {}
        self.log = ErrorMsgLog()  # 这行的所有错误
        self.split_frame_idx = split_frame_idx if split_frame_idx is not None else 0


    @abstractmethod
    def check_row_on_a9(self)->None:
        """a9只能通过annotation进行检查 将内容输出到log"""

    @abstractmethod
    def check_row_addtional(self)->None:
        """额外的内容检查 将内容输出到log """

    def check_row_func(self,check_mode="all"): #check分a9的
        if self.log is None or isinstance(self.log,ErrorMsgLog) ==False:
            raise Exception("请定义log 并且使用ErrorMsgLog的实例")
        else:
            if check_mode=="a9_check":
                self.check_row_on_a9()
                return self.log.error_list  ##直接返回对象
            elif check_mode == "all":
                self.check_row_on_a9()
                self.check_row_addtional()
                return self.log.fomart_error() ##做格式化
            else:
                self.log.create_error(msg="未定义的check_mode")
                return self.log.fomart_error

    @abstractmethod
    def parse_row_func(self)->(str,str,str): # 输入的是json字符串.注意在方法中只使用annotation不要使用其他的参数
        """实现自定义的 parse_row_func方法  应该返回相对保存路径，保存文件名和 json的内容字字符串"""


def handler_by_cls(context, event,cls):

    code = 0  # code 200代表成功
    msg = ""
    errors = []
    output={}
    save_path = ""
    from_cache = False

    try:
        payload = event.body  # 这个会自动转成dict
        assert payload["mode"] in {"a9_check","parse","save"},"模式不对"

        h = cls(
            file=payload.get("file",""),
            row_dict=payload["data"]["row_dict"],
            users_param=payload["data"].get("users_param",None),
            split_frame_idx = payload["data"].get("split_frame_idx",None)
        )

        if payload["mode"] =='a9_check':
            check_errors = h.check_row_func(check_mode="a9_check")
            errors.extend(check_errors)  # 将错误添加到errors.
            if len(errors) ==0:
                code =200
        else: ## 这里就是parse和save两种情况
            check_errors = h.check_row_func()
            errors.extend(check_errors)
            if len(errors)!=0:
                #存在错误则不再转换
                pass
            else:
                use_cache = payload.get("use_cache",False)# 是否使用缓存
                # 缓存使用 version + data字符串作为key  output作为值

                # outdir = ""  # 相对路径
                # outfilename="" # 文件名
                # json_str="" # 内容

                if use_cache == True:
                    v_key = context.user_data.version + json.dumps(payload["data"], ensure_ascii=False)
                    v_value = context.user_data.my_redis.get(v_key)
                    if v_value is not None:
                        outdir, outfilename, json_str = json.loads(v_value)  # 存在redis中的就是这三个
                        from_cache = True
                    else:
                        outdir, outfilename, json_str = h.parse_row_func()
                        context.user_data.my_redis.set(v_key,
                                                       json.dumps([outdir, outfilename, json_str], ensure_ascii=False),
                                                       ex=86400)
                else:
                    outdir, outfilename, json_str = h.parse_row_func()

                if payload["mode"] == "parse":
                    output = {
                        "outdir": outdir,
                        "outfilename": outfilename,
                        "json_str": json_str
                    }
                elif payload["mode"] == "save":  ### 根据传入的路径保存文件
                    save_dir = payload.get("save_dir","")
                    assert save_dir!= "", "保存路径不能为空"
                    save_path = os.path.join(save_dir, outdir)
                    output = {
                        "outdir": outdir,
                        "outfilename": outfilename,
                    }
                    os.makedirs(save_path, exist_ok=True)
                    with open(os.path.join(save_path, outfilename), "w", encoding='utf-8') as fw:
                        fw.write(json_str)

                ##成功后更新code
                code = 200
                msg = "success"

    except:
        errors.append(traceback.format_exc())
    finally:
        # print(save_path)
        return context.Response(
            body={
                "code": code,
                "msg": msg,
                "version":"v1",
                "output":output,
                "save_path":save_path,
                "from_cache":from_cache,
                "errors":errors
            },
            headers={},
            content_type='application/json',  # text/plain
            status_code=200
        )


# -*- coding: UTF-8 -*-

'''
Author: Henry Wang
Date: 2022-06-01 16:19
Short Description:

Change History:

'''

from peutils.transform.v1.base import *
import os

class LidarManifestConfig():
    def __init__(self):
        pass


class LidarManifestParse(CommonBaseMixIn):

    ### 继承session属性 用来读取url
    def __init__(self,base_url,config):
        self.raw_data = self.get_raw_data(os.path.join(base_url,"manifest.json"))
        self.config = config
        self.frame_length = self.get_frame_length()
        self.camera_list = self.get_camera_list()

    def get_frame_length(self):

        if self.raw_data["frames"].get("urls") is not None:
            f_num = len(self.raw_data["frames"].get("urls"))
        elif self.raw_data["frames"].get("data") is not None:
            f_num = len(self.raw_data["frames"].get("data"))
        else:
            raise Exception("MAINIFEST 未找到对应的帧数量")
        return f_num

    def get_camera_list(self):
        img_len = len(self.raw_data["images"])
        camera_list = [x["camera"] for x in self.raw_data["images"] ]
        # print(img_len,camera_list)
        if img_len != len(set(camera_list)):
            raise Exception(f"MAINIFEST 镜头数据解析错误 {camera_list}")

        return camera_list


# print(os.path.join("https://projecteng.oss-cn-shanghai.aliyuncs.com/3d_json/manifest/3615d49983984e7484ccd10dc8fc8f81/61d972a8910d7200e762ec55_9999_9999","manifest.json"))
if __name__ =="__main__":
    ## 单帧
    from pprint import pprint
    # mfst = LidarManifestParse(base_url="https://projecteng.oss-cn-shanghai.aliyuncs.com/3d_json/manifest/3615d49983984e7484ccd10dc8fc8f81/61d972a8910d7200e762ec55_9999_9999",
    #                      config =LidarManifestConfig())
    #
    # print(mfst.camera_list,mfst.frame_length)
    mfst = LidarManifestParse(
        base_url="https://projecteng.oss-cn-shanghai.aliyuncs.com/3d_json/manifest/23efe93357dd47a2877184a250bec73e/619c8ea93842dacdbf91b66e_351_400",
        config=LidarManifestConfig())

    print(mfst.camera_list, mfst.frame_length)
